package org.openxma.dsl.generator.impl

import java.util.LinkedHashMap
import java.util.Map
import javax.inject.Inject
import org.eclipse.emf.ecore.EObject
import org.eclipse.xtend.lib.Data
import org.openxma.dsl.pom.model.Content
import org.openxma.dsl.pom.model.CustomizeableField
import org.openxma.dsl.pom.model.XmadslPage
import org.eclipse.xtext.xbase.lib.Pair
import java.util.Set

class ArquillianTestCaseGenerator implements AutomationTestCaseGenerator {
	String ARQUILLIAN_TEST_PATH = "arquillian"
	
	String PAGE_OBJECT_PATH = "pageobjects"
	
	@Inject
    extension Types types
	
	@Inject
	extension JsfNames jsfNames
	
	def dispatch packageFolderPath(EObject eObject) {
        eObject.model?.name.replaceAll("\\.","/")
    }

	override getITCaseName(XmadslPage dslPage) {
		return dslPage.packageFolderPath + "/" + ARQUILLIAN_TEST_PATH + "/" + dslPage.name + "ITCase.java"
	}
	
	override generateTestCase(XmadslPage dslPage, String namespace) {'''
		package namespace.ARQUILLIAN_TEST_PATH;

		import namespace.ARQUILLIAN_TEST_PATH.PAGE_OBJECT_PATH.dslPage.name;
		import org.jboss.arquillian.drone.api.annotation.Drone;
		import org.jboss.arquillian.graphene.spi.annotations.Page;
		import org.jboss.arquillian.junit.Arquillian;
		import org.junit.Test;
		import org.junit.runner.RunWith; 
		import org.openqa.selenium.WebDriver;
		
		@RunWith(Arquillian.class)
		public class dslPage.nameITCase {
			
			@Drone
			WebDriver driver;
			
			@Page
			dslPage.name dslPage.bean;
			
			@Test
			public void testSample() {
				//implement test case
			}
		}
	'''
	}
	
	override getPageObject(XmadslPage dslPage, String namespace) {'''
		package namespace.ARQUILLIAN_TEST_PATH.PAGE_OBJECT_PATH;
		
		public class dslPage.name extends dslPage.nameGen {
		
		}
	'''
	}
	
	override getPageObjectName(XmadslPage dslPage, boolean generated) {
		return dslPage.packageFolderPath + "/" + ARQUILLIAN_TEST_PATH + "/" + PAGE_OBJECT_PATH + "/" + dslPage.name + (if (generated)  "Gen" else "") + ".java"
	}

	override getAbstractPageObject(XmadslPage dslPage, String namespace, Map<CharSequence,CharSequence> fieldNameMap) {'''
		package namespace.ARQUILLIAN_TEST_PATH.PAGE_OBJECT_PATH;
		
		import org.jboss.arquillian.drone.api.annotation.Drone;
		import org.jboss.arquillian.graphene.context.GrapheneContext;
		import org.openqa.selenium.WebDriver;
		import org.openqa.selenium.WebElement;
		import org.openqa.selenium.support.FindBy;
		
		import at.spardat.properties.XProperties;

		public abstract class dslPage.nameGen {
			
			@Drone
			protected WebDriver driver;
			
			createFields(fieldNameMap)
			
			public String getPageSource() {
				return GrapheneContext.getProxy().getPageSource();
			}
			
			public void open() {
				GrapheneContext.getProxy().get(getBaseUrl() + "/getFaceletPageName(dslPage, false)");
			}

			private String getBaseUrl()  {
				return XProperties.getNodeOfPackage("").get("arquillian.app.baseUrl", "http://localhost:9090/dsl-reference-jsf");
			}
		}
	'''
	}
	
	
	def dispatch CharSequence createFields(Map<CharSequence, CharSequence> fieldMap) {'''
		fieldMap.forEach(key, value | createField(key, value))
		FOR entry: fieldMap.entrySet
			createField(entry.key, entry.value)
		ENDFOR		
	'''
	}

	def dispatch CharSequence createField(CharSequence key, CharSequence value) { '''
		@FindBy(id="key")
		protected WebElement value;
		
	'''	
	}
	
	def Map getFields(Content content) {
		var fields = new LinkedHashMap<String, String>();
		for (item : content.eContents) {
			var field = getField(item)
			fields.put(field.key, field.value);
		}
		
		return fields;
	}
	
	def Pair<String, String> getField(EObject eObject) { 
		var pair = "" -> ""
		switch eObject {
			CustomizeableField : pair = eObject.attribute.name -> eObject.attribute.name + "1"
		}
		
		return pair
	}
}