package org.openxma.dsl.generator.impl

import com.google.inject.Singleton
import javax.inject.Inject
import org.eclipse.emf.common.util.EList
import org.openxma.dsl.core.model.ConditionsBlock
import org.openxma.dsl.pom.PomFactory
import org.openxma.dsl.pom.model.Command
import org.openxma.dsl.pom.model.Component
import org.openxma.dsl.pom.model.XmadslPage

import static org.openxma.dsl.generator.lib.ImportAdapter.*
import org.openxma.dsl.pom.model.Content
import org.eclipse.emf.ecore.EObject
import org.openxma.dsl.pom.model.Text
import org.openxma.dsl.pom.model.ListBox
import org.openxma.dsl.pom.model.TableCombo
import org.openxma.dsl.pom.model.Combo
import org.openxma.dsl.pom.model.CustomizeableField
import org.openxma.dsl.pom.model.Table
import org.openxma.dsl.pom.model.TabFolder
import org.openxma.dsl.pom.model.TabPage
import org.openxma.dsl.pom.model.ReferencedXMAComposite
import org.openxma.dsl.pom.model.CheckBox
import com.google.common.base.CaseFormat
import org.eclipse.emf.ecore.util.EcoreUtil
import org.openxma.dsl.pom.model.XmadslComposite
import org.openxma.dsl.pom.model.Button


@Singleton
class BackingBeanImpl implements BackingBean {
	@Inject
	extension Names
	@Inject
	extension JsfNames
	@Inject
	extension JsfEvents
	@Inject
	extension JsfData
	@Inject
	extension ExpressionCompiler
	
	override CharSequence toComponentBackingBean(String namespace, Component it) { '''
package namespace.backingbean;

addImportsHere(it)

/** 
 * Backing Bean for Component xmaComponent.namClass 
 */
@addImport("org.springframework.stereotype.Component")("xmaComponent.namClass.toFirstLower")
@addImport("org.springframework.context.annotation.Scope")("session")
public class xmaComponent.namClass extends xmaComponent.namClassGen {
}
'''
	}
	
	override CharSequence toBackingBean(String namespace, XmadslPage it) { '''
package namespace.backingbean;

addImportsHere(it)

/** 
 * Backing Bean for Page name 
 */
@addImport("org.springframework.stereotype.Component")("name.toFirstLower")
@addImport("org.springframework.context.annotation.Scope")("access")
public class name extends nameGen {
}
'''
	}
	

	override CharSequence toAbstractComponentBackingBean(String namespace, Component it) {'''
package namespace.backingbean;
addImportsHere(it)
/** 
 * Abstract Backing Bean Base for Component xmaComponent.namClass 
 */
public abstract class xmaComponent.namClassGen extends addImport("org.openxma.dsl.platform.jsf.beans.ComponentBackingBean") {
	// resource bundle
	protected addImport("java.util.ResourceBundle") bundle = ResourceBundle.getBundle("xmaComponent.namPackageAsPath + "/" + xmaComponent.namClass", addImport("javax.faces.context.FacesContext").getCurrentInstance().getViewRoot().getLocale());
	
	dependencies.toInjectedDependencies
	componentProperties.toObjectPropertyVars
	
	dataobjects.toDataObjectVars
	if (events != null) events.toLifecycleMethods else PomFactory::eINSTANCE.createEventMappingList.toLifecycleMethods
	
    // commands
	commands.toAbstractCommands
	
	
    FOR compProperty : componentProperties
    	compProperty.toGetterAndSetter
	ENDFOR

	FOR dataObject : dataobjects
		dataObject.toGetterAndSetter
	ENDFOR
}
	''' 
	}
	override CharSequence toAbstractBackingBean(String namespace, XmadslPage it, Component comp) {'''
package namespace.backingbean;
addImportsHere(it)
/** 
 * Abstract Backing Bean Base for Page name 
 */
public abstract class nameGen extends addImport("org.openxma.dsl.platform.jsf.beans.PageBackingBean") {
	
	// client id constants
	content?.toFaceletIdConstants
	
	
	@addImport("org.springframework.beans.factory.annotation.Autowired")
	protected comp.xmaComponent.namClass component;
	
	public comp.xmaComponent.namClass getTypedComponent() {
		return component;
	}
	
	dependencies.toInjectedDependencies
	pageProperties.toObjectPropertyVars
	
	
    // data objects
    dataobjects.toDataObjectVars
    
    // jsf data models
    content?.toJsfDataModel
    
    // init data model
    protected void dataModelInit() {
	    content?.toInitCode
    }
    
    // lifecycle events
    it.toLifecycleMethods
    if (events != null) events.toLifecycleMethods else PomFactory::eINSTANCE.createEventMappingList.toLifecycleMethods
    
    // commands
	commands.toAbstractCommands
    
    // conditions
    conditionsBlock?.toConditions

    FOR pageProperty : pageProperties
    	pageProperty.toGetterAndSetter
	ENDFOR

	FOR dataObject : dataobjects
		dataObject.toGetterAndSetter
	ENDFOR
	
	public void handlePreRenderView(addImport("javax.faces.event.ComponentSystemEvent") event){
		processConditions();
		if (!addImport("javax.faces.context.FacesContext").getCurrentInstance().isPostback()) {
			customizeView(FacesContext.getCurrentInstance().getViewRoot());
		}
	}
	
	protected void customizeView(addImport("javax.faces.component.UIViewRoot") viewRoot) {
		//by default do nothing, override to add behaviour
	}
	protected nameGen getContext() {
		return this;
	}
	protected void processConditions() {
		conditionsBlock?.toConditionsProcessing
	}

}
'''		
	}
	
	
	
	override CharSequence toConditions(ConditionsBlock conditionsBlock) {'''
    	FOR statusFlag : conditionsBlock?.statusFlags
    		private boolean statusFlag.name;
    		statusFlag.toGetterAndSetter()
    	ENDFOR
'''
	}
	def CharSequence toConditionsProcessing(ConditionsBlock conditionsBlock) {'''
    	FOR statusFlag : conditionsBlock?.statusFlags
    		setstatusFlag.name.toFirstUpper(statusFlag.expression.compile);   
    	ENDFOR
'''
	}
	def CharSequence toFaceletIdConstants(EObject object) { 
		val Iterable<EObject> allElements = EcoreUtil::getAllContents(object,false).toIterable;
	'''
		object.toFaceletIdConstant
		
		FOR element : allElements
			(element as EObject).toFaceletIdConstant
		ENDFOR
'''		
	}
	def CharSequence toFaceletIdConstant(EObject object) {
		val id = object.toFaceletId
		if (id != null) {
			val name = name(object)
			if (name != null) 			
				'''public static String facetletIdConstantName(name) = "object.parentPage.toFaceletFormId:id";'''
			else
				''''''
		}
		else
			''''''
	}
	
	
	def CharSequence facetletIdConstantName(String name) {
		CaseFormat::LOWER_CAMEL.to(CaseFormat::UPPER_UNDERSCORE,name) + "_ID"
	}
	def String name(EObject object) {
		switch object {
			Text : { (object as Text).name }
			ListBox : { (object as ListBox).name }
			TableCombo : { (object as TableCombo).name }
			Table : { (object as Table).name }
			Combo : { (object as Combo).name }
			Button : { (object as Button).name }
			// TODO: field reference has no name, but has to be unique
//			CustomizeableField : { (object as CustomizeableField).attribute.name }
			XmadslComposite : { (object as XmadslComposite).name }
			ReferencedXMAComposite : { (object as ReferencedXMAComposite).name }
			TabFolder : { (object as TabFolder).name }
			TabPage : { (object as TabPage).name }
			CheckBox : { (object as CheckBox).name }
			ListBox : { (object as ListBox).name }
			default : null
		}
	}
	
	
	override CharSequence toAbstractCommands(EList<Command> commands) { '''
	    FOR command : commands
			IF !command.clientOnly
    public abstract void command.name();
    		ENDIF
    	ENDFOR
	'''
	}
	
 
}