package org.openxma.dsl.generator.impl

import javax.inject.Inject
import org.eclipse.emf.ecore.EObject
import org.openxma.dsl.core.model.AdditiveExpr
import org.openxma.dsl.core.model.AdditiveRights
import org.openxma.dsl.core.model.AtomicExpr
import org.openxma.dsl.core.model.Call
import org.openxma.dsl.core.model.CondANDExpr
import org.openxma.dsl.core.model.CondANDRights
import org.openxma.dsl.core.model.CondORExpr
import org.openxma.dsl.core.model.CondORRights
import org.openxma.dsl.core.model.EqualityExpr
import org.openxma.dsl.core.model.FalseLiteral
import org.openxma.dsl.core.model.IntLiteral
import org.openxma.dsl.core.model.Literal
import org.openxma.dsl.core.model.MultiplicativeExpr
import org.openxma.dsl.core.model.MultiplicativeRights
import org.openxma.dsl.core.model.ParenExpr
import org.openxma.dsl.core.model.RelationalExpr
import org.openxma.dsl.core.model.SimpleType
import org.openxma.dsl.core.model.StatusFlag
import org.openxma.dsl.core.model.StringLiteral
import org.openxma.dsl.core.model.TrueLiteral
import org.openxma.dsl.core.model.Variable
import org.openxma.dsl.core.model.VariableAccess
import org.openxma.dsl.core.model.XmadslVariable
import org.openxma.dsl.dom.model.ApplicationSession
import org.openxma.dsl.dom.model.Attribute
import org.openxma.dsl.dom.model.Property

import static extension org.eclipse.emf.ecore.util.EcoreUtil.*
import static extension org.openxma.dsl.common.extensions.EObjectPropertiesAdapter.*
import static extension org.openxma.dsl.generator.helper.ExpressionExtension.*
import static extension org.openxma.dsl.generator.helper.SimpleTypeExtension.*
import org.openxma.dsl.pom.model.ObjectProperty

class FaceletExpressionCompiler {
	
	@Inject
	extension Features
	@Inject
	extension JsfNames
	@Inject
	extension Names

	def dispatch String compile(EObject it) {}
	
	def dispatch String compile(EqualityExpr it) {
		var result = left.compile
		pushOperator(op.toString)
		if (right != null) {
			result = result + right.compile
		}
		result
	}

	def dispatch String compile(CondORExpr it) {
		compileLeftThenRight(left,rights)
	}
	
	def dispatch String compile(CondANDExpr it) {
		compileLeftThenRight(left,rights)
	}
	
	def dispatch String compile(CondORRights it) {
		pushOperator("||")
		right.compile
	}
	
	def dispatch String compile(CondANDRights it) {
		pushOperator("&&")
		right.compile
	}
	
	def dispatch String compile(RelationalExpr it) {
		var result = left.compile
		pushOperator(op.toString)
		if (right != null) {
			result = result + right.compile
		}
		result
	}
	
	def dispatch String compile(AdditiveExpr it) {
		compileLeftThenRight(left,rights)
	}

	def dispatch String compile(AdditiveRights it) {
		pushOperator(op.toString())
		right.compile
	}
	
	def dispatch String compile(MultiplicativeExpr it) {
		compileLeftThenRight(left,rights)
	}
	
	def dispatch String compile(MultiplicativeRights it) {
		pushOperator(op.toString())
		right.compile
	}
	
	def dispatch String compile(XmadslVariable it) {
		switch (access) {
			case VariableAccess::NULL : reference.compile(it)
			case VariableAccess::VALUE : reference.compile(it)
			case VariableAccess::HAS_VALUE : beginAtomicExpr(it) + "(" + reference.compile(it) + "!=null)"
			case VariableAccess::IS_VALID : "ToDo"
		}
	}
	
	def dispatch String compile(Property it, Variable variable) {
		'''beginAtomicExpr(variable)nameendAtomicExpr(variable)'''
	}
	
	def dispatch String compile(StatusFlag it, Variable variable) {
		'''beginAtomicExpr(variable)parentPage.bean.nameendAtomicExpr(variable)'''
	}
	
	def dispatch String compile(ObjectProperty it, Variable variable) {
		'''beginAtomicExpr(variable)parentPage.bean.nameendAtomicExpr(variable)'''
	}
	
	def dispatch String compile(Call it) {
		'''beginAtomicExpr(it)getContext().function.name(expr.compile)endAtomicExpr(it)'''
	}
	
	def dispatch String compile(ParenExpr it) {
		val result = '''beginAtomicExpr(it)(expr.compile)endAtomicExpr(it)'''
		if (rootContainer.getProperty("parenExpr")!=null) {
			rootContainer.removeProperty("parenExpr")	
		}
		result.toString
	}
	
	def dispatch String compile(Literal it) {
		''''''
	}
	
	def dispatch String compile(IntLiteral it) {
		 '''beginAtomicExpr(it)numberendAtomicExpr(it)'''
	}
	
	def dispatch String compile(StringLiteral it) {
		 '''beginAtomicExpr(it)'string'endAtomicExpr(it)'''
	}
	
	def dispatch String compile(TrueLiteral it) {
		 '''beginAtomicExpr(it)trueendAtomicExpr(it)'''
	}
	
	def dispatch String compile(FalseLiteral it) {
		 '''beginAtomicExpr(it)falseendAtomicExpr(it)'''
	}
	
	
	def dispatch String compile(Attribute it,  Variable variable) {
		if (dataType instanceof SimpleType && isPrimitiveWrapperType(dataType as SimpleType)) {
			'''beginAtomicExpr(variable)getValue(eContainer.toVariableName,"name",type.qualifiedName.class)endAtomicExpr(variable)'''	
		} else {
			'''beginAtomicExpr(variable)eContainer.toVariableName.getAccessorMethodName()()endAtomicExpr(variable)'''	
		}
	}
	
	def dispatch String beginAtomicExpr(EObject it, AtomicExpr atomicExpr) {
		convertOperator 
	}
	
	def dispatch String beginAtomicExpr(org.openxma.dsl.dom.model.Property it, AtomicExpr atomicExpr) {
		if (!isPrimitiveWrapperType(type)) {
			objectExpressionValue(atomicExpr)  
		} else {
			convertOperator
		}
	}
	
	def dispatch String beginAtomicExpr(Attribute it, AtomicExpr atomicExpr) {
		if (!(dataType instanceof SimpleType) || !isPrimitiveWrapperType(dataType as SimpleType)) {
			objectExpressionValue(atomicExpr)  	
		} else {
			convertOperator
		}
	}
	
	def dispatch String beginAtomicExpr(StringLiteral it, AtomicExpr atomicExpr) {
		if (!(eContainer instanceof Call)) {
			objectExpressionValue(atomicExpr)
		} 
	}
	
	def dispatch String beginAtomicExpr(ParenExpr it, AtomicExpr atomicExpr) {
		if (!expr.isPrimitiveWrapperType()) {
			'''objectExpressionValue(it)atomicExpr.rootContainer.setProperty("parenExpr","true")'''   
		} else {
			convertOperator	
		}
	}

	def dispatch String beginAtomicExpr(Call it, AtomicExpr atomicExpr) {
		if (!function.resultType.isPrimitiveWrapperType()) {
			objectExpressionValue(it) 
		} else {
			convertOperator	
		}
	}
	
	def String objectExpressionValue(EObject it, AtomicExpr atomicExpr) {
		convertOperator
	}
	
	def String endAtomicExpr(EObject it, AtomicExpr atomicExpr) {
		if (atomicExpr.rootContainer.hasProperty("equals")) {
			atomicExpr.rootContainer.removeProperty("equals")
			return ")"	
		}
		return ""
	}
	
	
	def String compileLeftThenRight(EObject left,Iterable<? extends EObject> rights) {
		val result = new StringBuilder(left.compile)
		rights.forEach(e|result.append(e.compile))
		result.toString
	}
	
	def String convertOperator(EObject it) {
		val op = popOperator() 
		switch op {
			case "==" : return " eq "
			case "!=" : return " ne "
			case "<" : return " lt "
			case ">" : return " gt "
			case "<=" : return " le "
			case ">=" : return " ge "
		}
		op
	}
}