package org.openxma.dsl.generator.impl

import com.google.inject.Singleton
import java.beans.Introspector
import java.beans.PropertyDescriptor
import java.util.ArrayList
import java.util.LinkedHashMap
import java.util.Map
import javax.inject.Inject
import org.apache.commons.logging.Log
import org.eclipse.emf.common.util.EList
import org.eclipse.emf.ecore.EObject
import org.openxma.dsl.core.model.EnumType
import org.openxma.dsl.core.model.FalseLiteral
import org.openxma.dsl.core.model.TrueLiteral
import org.openxma.dsl.dom.model.ReadOnlyFlag
import org.openxma.dsl.dom.model.RequiredFlag
import org.openxma.dsl.pom.model.AttachmentProperty
import org.openxma.dsl.pom.model.Button
import org.openxma.dsl.pom.model.CheckBox
import org.openxma.dsl.pom.model.Combo
import org.openxma.dsl.pom.model.Command
import org.openxma.dsl.pom.model.CommandImplementation
import org.openxma.dsl.pom.model.Component
import org.openxma.dsl.pom.model.ComposeData
import org.openxma.dsl.pom.model.Content
import org.openxma.dsl.pom.model.CustomizeableField
import org.openxma.dsl.pom.model.EnabledFlag
import org.openxma.dsl.pom.model.EventMappingList
import org.openxma.dsl.pom.model.EventType
import org.openxma.dsl.pom.model.FieldPart
import org.openxma.dsl.pom.model.FieldReference
import org.openxma.dsl.pom.model.GuiElementEventMapping
import org.openxma.dsl.pom.model.IGuiElementWithEvent
import org.openxma.dsl.pom.model.IReferenceableByStatementVariable
import org.openxma.dsl.pom.model.InvokeStatement
import org.openxma.dsl.pom.model.Invokeable
import org.openxma.dsl.pom.model.JoinDirection
import org.openxma.dsl.pom.model.Label
import org.openxma.dsl.pom.model.LabelText
import org.openxma.dsl.pom.model.ListBox
import org.openxma.dsl.pom.model.ObjectProperty
import org.openxma.dsl.pom.model.Page
import org.openxma.dsl.pom.model.RadioButton
import org.openxma.dsl.pom.model.ReferencedXMAComposite
import org.openxma.dsl.pom.model.Seperator
import org.openxma.dsl.pom.model.SetOfGuiElements
import org.openxma.dsl.pom.model.Statement
import org.openxma.dsl.pom.model.TabFolder
import org.openxma.dsl.pom.model.TabPage
import org.openxma.dsl.pom.model.Table
import org.openxma.dsl.pom.model.TableColumn
import org.openxma.dsl.pom.model.TableCombo
import org.openxma.dsl.pom.model.Text
import org.openxma.dsl.pom.model.Tree
import org.openxma.dsl.pom.model.VisibleFlag
import org.openxma.dsl.pom.model.XmadslComposite
import org.openxma.dsl.pom.model.XmadslPage

import static org.apache.commons.logging.LogFactory.*
import static org.openxma.dsl.generator.helper.GeneratorConfigurationExtension.*
import org.openxma.dsl.pom.model.IElementWithLayoutData
import org.openxma.dsl.pom.model.FieldFlag

@Singleton
class FaceletImpl implements Facelet {
	
	val Log logger = getLog(getClass().getName());
	
	@Inject
	extension Files
	@Inject
	extension JsfNames
	@Inject
	extension ExpressionLanguageImpl
	@Inject
	extension FaceletDataImpl
	@Inject
	extension FaceletLayoutImpl
	@Inject
	extension FaceletExpressionCompiler
	
	var itemIndex = 0;
	
	val Boolean createIdentifiers = false;
	
	var resourceBundleName = "msgs";
	
	var formName = "";
	
	Map<CharSequence, CharSequence> resourceProperties = new LinkedHashMap<CharSequence, CharSequence>(); 
	
	Map<CharSequence, CharSequence> clientIds = new LinkedHashMap<CharSequence, CharSequence>(); 
	
	override getPageResourceProperties(EObject composite) {
		resourceProperties
	}
	
	override getFieldClientIds(EObject composite) {
		clientIds
	}
	

	def dispatch CharSequence createContents(XmadslPage page) {
		clientIds.clear()
		resourceBundleName = page.component.xmaComponent.namClass.toLowerCase + "msgs"
		formName = page.name + "Form"
	'''
<?xml version="1.0" encoding="UTF-8"?> 
<!DOCTYPE html
        PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
        "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml"
	xmlns:f="http://java.sun.com/jsf/core"
	xmlns:h="http://java.sun.com/jsf/html"
	xmlns:ui="http://java.sun.com/jsf/facelets"
	xmlns:p="http://primefaces.org/ui"
	xmlns:fn="http://java.sun.com/jsp/jstl/functions"
	xmlns:pe="http://primefaces.org/ui/extensions">
<head>
</head>
<body>
<ui:composition template="${template}">
		<ui:param name="title" value="processResourceBundleProperty(page, page.labelText?.label)" />
		<ui:param name="dialogId" value="page.name.toFirstLower" />
		<ui:param name="dialogWidgetVar" value="page.dialogWidgetVar" />
		<ui:define name="dialogs">
			FOR invokeablePage : page.invokablePages
			<ui:include src="/invokeablePage.faceletPageFileName(false)">
				<ui:param name="template" value="/templates/dialogTemplate.xhtml" />
			</ui:include>
			ENDFOR
		</ui:define>
		<ui:define name="genscripts">
			<script type="text/javascript">
				if (!App.page.component.JSClass) {
					App.page.component.JSClass = Xma.Component.extend( {
						page.component.commands?.toUiCommandPrototypes
						getMainPage : function() { return App.page.component.mainPage.widgetVar; }
					});
					App.page.component.widgetVar = new App.page.component.JSClass();
				}
				App.page.JSClass = Xma.Page.extend({
					page.commands?.toUiCommandPrototypes
					invoke : function(callback) { this.callback = callback; page.name_invokeCmd(); },
					getDialogWidget : 	function() { return page.dialogWidgetVar;}
				});	
				App.page.widgetVar = new App.page.JSClass();
			</script>	
		</ui:define>
		<ui:define name="center">
			<f:loadBundle basename="page.component.xmaComponent.namPackage.page.component.xmaComponent.namClass" var="resourceBundleName"/>
			<h:form id="page.toFaceletFormId">
				<f:event listener="#{page.bean.handlePreRenderView}" type="preRenderView" />
				page.content?.createContents
				<p:remoteCommand action="#{page.bean.invoke}" process="page.processedInputsList" update="@form" id="invoke" name="page.name_invokeCmd" oncomplete="App.page.widgetVar.showDialog();">
					<p:resetInput target="page.nameForm" />
				</p:remoteCommand>
				page.commands?.toRemoteCommands
				page.pageProperties?.toRemoteProperties
				<ui:insert name="additionalWidgets" />
				IF useTooltipForValidationMessage
				<pe:tooltip global="true" myPosition="left center" atPosition="right center" forSelector=".ui-state-error"/>
                ENDIF
			</h:form>
		</ui:define>

	</ui:composition>
</body>
</html>
'''
	}
	
	def addClientIdForField(CharSequence name, CharSequence clientId) {
		clientIds.put(clientId, name);
	}
	
	def addClientIdForField(CharSequence name) {
		clientIds.put(name, name);
	}
	
	def CharSequence getProcessedInputsList(XmadslPage page) {
		if (page.pageProperties == null || page.pageProperties.empty)
			return "@none"
		'''FOR prop : page.pageProperties SEPARATOR ','prop.nameENDFOR'''
	}
	
	def Iterable<XmadslPage> getInvokablePages(XmadslPage page) {
		val implementations = page.commands.map([implementation]).<CommandImplementation>toList.filterNull
        val statements = implementations.map([statements.filter(typeof(InvokeStatement))]).flatten
        val invokeables = statements.map([invokeAble]).<Invokeable>toSet
		invokeables.filter(typeof(XmadslPage))
	}
	
	
	def CharSequence toRemoteProperties(EList<ObjectProperty> properties) { '''
	    FOR property : properties
	<p:inputText value="#{property.parentPage.bean.property.name}" style="display:none;" id="property.name" widgetVar="property.widgetVar" />
    	ENDFOR
	'''
	}
	def CharSequence toRemoteCommands(EList<Command> commands) { '''
	    FOR command : commands.filter(c|!c.name.equals("init"))
			IF !command.clientOnly && !hasEventMappings(command)
    			<p:remoteCommand action="#{command.parentPage.bean.command.name}" update="@form" id="command.name" name="command.widgetVar" />
    		ENDIF
    	ENDFOR
	'''
	}
	
	
	
	def dispatch CharSequence createContents(Table table) { 
	addClientIdForField(table.name, formName + ":" + table.name)	
	'''
		    <p:dataTable id="table.toFaceletId"
		    IF table.multiselectionFlag?.multiSelection instanceof TrueLiteral || table.checkSelection instanceof TrueLiteral 
		     	selectionMode="multiple" selection="#{table.modelProperty.selectedItems}"
		     	ELSE
		     	selectionMode="single" selection="#{table.modelProperty.selectedItem}"
		     	ENDIF
		     	widgetVar="table.widgetVar"
		    value="#{table.modelProperty}" var="item"
		    rowKey="#{item.oid}"
			    IF table.pageable instanceof TrueLiteral
			    paginator="true" rows="10" paginatorPosition="top" paginatorTemplate="{CurrentPageReport}  {FirstPageLink} {PreviousPageLink} {PageLinks} {NextPageLink} {LastPageLink}"
			    ENDIF
			    resizableColumns="true" >
			    table.createEventMappingsForControl
		    FOR column : table.columns
			    column.createContents
		    ENDFOR
		    </p:dataTable>
'''	
	}
	
	def dispatch CharSequence createContents(Tree tree) { '''
		    <p:tree id="tree.toFaceletId" 
		     	selectionMode="single" widgetVar="tree.nameWidgetVar"
			    value="#{tree.rootNodeProperty}" var="node"
			    selection="#{tree.selectedNodeProperty}" tree.disabledFlag>
			    tree.createEventMappingsForControl
			    <p:treeNode>
			    	<h:outputText value="#{node}" />
			    </p:treeNode>
		    </p:tree>
'''	
	}
	
	def dispatch CharSequence createEventMapping(IGuiElementWithEvent control, GuiElementEventMapping mapping) { '''
		<!-- error not control event mapping for control type control.getClass().name -->
	'''
	}
	def dispatch CharSequence createEventMapping(Table table, GuiElementEventMapping mapping) {
		switch mapping.event {
			case EventType::ON_SELECTION : '''<p:ajax event="rowSelect" table.addFunctionCommands("listener", "onstart") update="@form" />'''
			case EventType::ON_DOUBLE_CLICK : '''<p:ajax event="rowDblselect" table.addFunctionCommands("listener", "onstart") update="@form" />'''
			default : ''''''
		}
	}
	def CharSequence createEventMappingsForControl(IGuiElementWithEvent control) { '''
		FOR mapping : control.eventMappingsForControl
			control.createEventMapping(mapping)
		ENDFOR	
	'''
	}
	def Iterable<GuiElementEventMapping> getEventMappingsForControl(IGuiElementWithEvent control) {
		control.parentPage.events.mapping.filter(typeof(GuiElementEventMapping)).filter[m | m.control == control]
	}
	
	def String processResourceBundleProperty(EObject eObject, CharSequence text) {
		var s = new String();
		
		switch eObject {
			CustomizeableField : s = getLabelForCustomField(eObject)
			Button : s = eObject.parentPage.name + "." + eObject.name
			TableColumn : 
				if (eObject.eContainer instanceof Table) 
					s =  eObject.parentPage.name + "." + (eObject.eContainer as Table).name + "_" + (eObject.columnName ?: eObject?.attribute?.name)
				else
					s =  eObject.parentPage.name + "." + (eObject.eContainer as TableCombo).name + "_" + (eObject.columnName ?: eObject?.attribute?.name)
			Label : s = eObject.parentPage.name + "." + (eObject.name ?: "label" + "_" + itemIndex)
			CheckBox : s = eObject.parentPage.name + "." + (eObject.name ?: "checkbox" + "_" + itemIndex)
			TabPage : s = eObject.parentPage.name + "." + eObject.name
			XmadslComposite : s = eObject.parentPage.name + "." + eObject.name
			XmadslPage : s = (eObject as XmadslPage).name + "." + "title"
		}
		
		resourceProperties.put(s, text)
		return "${" + resourceBundleName + "['" + s + "']}";
	}
	
	def String getLabelForCustomField(CustomizeableField field) {
		if (field.eContainer.eContainer instanceof TabPage) {
			var contatiner = field.eContainer.eContainer as TabPage 
			Character::toUpperCase(contatiner.name.charAt(0)) + contatiner.name.substring(1) + "." + field.object.name + "_" + field.attribute.name
		} else {
			field.parentPage.name + "." + field.object.name + "_" + field.attribute.name
		}
	} 

	
	def dispatch CharSequence createContents(TableColumn column) { '''
			    <p:column headerText="processResourceBundleProperty(column, column.labelText.toEncodedHtml)"
			    	IF "styling".equals("styling") && column.attribute != nullstyleClass="column.attribute.name"ENDIF
				    IF (column.feature != null && column.feature.attributeHolder != null)sortBy="#{column.columnValueExpression}">
				    <h:outputText value="#{column.columnValueExpression}">
				    	column.feature.attributeHolder.attribute.type.createConverter
				    </h:outputText>
				    ELSE
				    >
				    <h:outputText value="hardcodedtext" />
				    ENDIF
			    </p:column>
''' 
	}

	
	def dispatch CharSequence createContents(Text text) { 
		addClientIdForField(text.name, formName + ":" + text.toFaceletId)
		'''
		    <p:inputText id="text.toFaceletId" value="#{text.parentPage.bean.text.nameText}" text.disabledFlag />
		''' 
	}

	def dispatch CharSequence createContents(Combo combo) { 
		addClientIdForField(combo.name, formName + ":" + combo.toFaceletId)
	'''
		 	<p:selectOneMenu id="combo.toFaceletId" value="#{combo.parentPage.bean.combo.nameCombo}" combo.disabledFlag>
		 		<f:selectItem itemLabel="Select value" itemValue="" />
		        <f:selectItems value="#{combo.selectItemsProperty}" var="item" itemLabel="#{item.name}" itemValue="#{item}" />
		    </p:selectOneMenu>'''
	}
	
	def dispatch CharSequence createContents(TableCombo combo) { 
		addClientIdForField(combo.name, formName + ":" + combo.toFaceletId)
	'''
		 	<p:selectOneMenu id="combo.toFaceletId" value="#{combo.parentPage.bean.combo.nameTableCombo}" var="item" combo.disabledFlag>
		 		<f:selectItem itemLabel="Select value" itemValue="" />
		        <f:selectItems value="#{combo.listProperty}" var="item" itemLabel="#{item.name}" itemValue="#{item}" />
		        FOR col : combo.columns
		        	col.createContents
		        ENDFOR
		    </p:selectOneMenu>'''
	}
	
	def dispatch CharSequence createContents(ListBox listbox) { 
		addClientIdForField(listbox.name, formName + ":" + listbox.toFaceletId)
	'''
		 	<p:selectOneListbox id="listbox.toFaceletId" value="#{listbox.parentPage.bean.listbox.nameListBox}" listbox.disabledFlag>
		 		<f:selectItem itemLabel="Select value" itemValue="" />
		        <f:selectItems value="#{listbox.selectItemsProperty}" var="item" itemLabel="#{item.name}" itemValue="#{item}" />
		    </p:selectOneListbox>'''
	}
	
	def dispatch CharSequence createContents(RadioButton radio) { '''
		 	<ui:remove>Not supported: Radiobutton! </ui:remove>'''
	}
	
	
	def CharSequence uniqueIdentifierAtribute(XmadslComposite composite) {
		if (composite.name != null && createIdentifiers) 
			return '''id="composite.toFaceletId"'''  
		""
	}
	def CharSequence uniqueIdentifierAtribute(TabFolder folder) {
		if (folder.name != null && createIdentifiers)
			return '''id="folder.toFaceletId"'''  
		""
	}
	
	def dispatch CharSequence createContents(TabFolder tabFolder) { '''
	    <p:tabView tabFolder.uniqueIdentifierAtribute>
	    FOR item : tabFolder.eContents
		    item.createContents
	    ENDFOR
	    </p:tabView>
''' 
	}
	
	def dispatch CharSequence createContents(TabPage tabPage) { 
		addClientIdForField(tabPage.name)
		'''
			IF isCustomizable(tabPage)<ui:insert name="getItemName(tabPage)">ENDIF
			<p:tab id="tabPage.toFaceletId" title="processResourceBundleProperty(tabPage, tabPage.labelText?.string)">
				FOR item : tabPage.eContents
					beforeChildItem(false, false, 0, item)
					item.createContents
					afterChildItem(false, false, 0, item)
				ENDFOR
			</p:tab>
			IF isCustomizable(tabPage)</ui:insert>ENDIF
		''' 
	}
	
	def dispatch CharSequence createContents(Button button) { 
		addClientIdForField(button.name, formName + ":" + button.toFaceletId)
		'''<p:commandButton id="button.toFaceletId" value="processResourceBundleProperty(button, button.labelText.label)" button.addFunctionCommands("actionListener", "onclick") IF !hasEventMappings(button, false)type="button"ELSEupdate="@form" ENDIF button.buttonStyle  button.disabledFlag/>'''
	}

	def dispatch CharSequence createContents(Seperator seperator) { '''
		    <p:separator />
''' 
	}
	
	def dispatch CharSequence createContents(FieldReference field) { '''
		    <h:field attribute="field.attribute.name"/>
''' 
	}
	
	def dispatch CharSequence createContents(Label label) {	'''			
		    <h:outputLabel value="processResourceBundleProperty(label, label.labelText.label.toEncodedHtml)" />
'''	
	}
	
	def dispatch CharSequence createContents(CheckBox checkBox) { 
	addClientIdForField(checkBox.name, formName + ":" + checkBox.toFaceletId)
	'''
	<p:selectBooleanCheckbox id="checkBox.toFaceletId" value="#{checkBox.parentPage.bean.checkBox.nameSelected}" checkBox.disabledFlag />
	<p:outputLabel value="processResourceBundleProperty(checkBox, checkBox.labelText.label.toEncodedHtml)" for="checkBox.toFaceletId" />
	''' }
		
	def dispatch CharSequence createContents(LabelText label) {	'''
		IF label.eContainer instanceof TabPage && (label.eContainer as TabPage).labelText == label
		ELSE
		    <h:outputLabel value="label.label.toEncodedHtml" />
		ENDIF
'''	
	}
	def dispatch CharSequence createContents(CustomizeableField field) {	
		addClientIdForField(field.attribute.name, formName + ":" + field.toFaceletId)
	'''
		IF field.parts == null || field.parts.size == 0 || field.parts.exists(fps|fps.part == FieldPart::LABEL)
				renderStartColumn(field)
				<p:outputLabel value="processResourceBundleProperty(field, field.labelText.toEncodedHtml)" for="field.toFaceletId"/>
				renderEndColumn(field)
		ENDIF
		IF field.parts == null || field.parts.size == 0 ||  field.parts.exists(fps|fps.part == FieldPart::CONTROL)
				renderStartColumn(field)
			    IF field.fieldFlags.exists(f|f instanceof VisibleFlag && f.expression instanceof FalseLiteral)
			    <p:inputText id="field.toFaceletId" value="#{field.fieldValueExpression}" style="display:none;" />
			    ELSEIF field.attribute.type.dataType instanceof EnumType
			    <p:selectOneMenu id="field.toFaceletId" value="#{field.fieldValueExpression}" field.disabledFlag field.requiredFlag IF useTooltipForValidationMessagetitle="#{component.valid ? '' : facesContext.getMessageList(component.getClientId())[0].summary}"ENDIF>
			    	<f:selectItem itemLabel="Select value" itemValue="" />
			        <f:selectItems value="#{field.selectItemsProperty}" var="item" itemLabel="#{item.name}" itemValue="#{item}" />
			    </p:selectOneMenu>
			    ELSEIF field.attribute.type.validatorReferenceDefinition.i18nValidator
			    <p:selectOneMenu id="field.toFaceletId" value="#{field.fieldValueExpression}" field.disabledFlag field.requiredFlag IF useTooltipForValidationMessagetitle="#{component.valid ? '' : facesContext.getMessageList(component.getClientId())[0].summary}"ENDIF>
			    	<f:selectItem itemLabel="Select value" itemValue="" />
			        <f:selectItems value="#{field.selectItemsProperty}" var="item" itemLabel="#{item.name}" itemValue="#{item}" />
			    </p:selectOneMenu>
			    ELSEIF field.attribute.type.validatorReferenceDefinition.booleanValidator
			    <p:selectBooleanCheckbox id="field.toFaceletId" value="#{field.fieldValueExpression}" field.disabledFlag field.requiredFlag IF useTooltipForValidationMessagetitle="#{component.valid ? '' : facesContext.getMessageList(component.getClientId())[0].summary}"ENDIF/>
			    ELSEIF field.attribute.type.validatorReferenceDefinition.dateValidator
			    <p:calendar id="field.toFaceletId" value="#{field.fieldValueExpression}" field.disabledFlag field.requiredFlag IF useTooltipForValidationMessagetitle="#{component.valid ? '' : facesContext.getMessageList(component.getClientId())[0].summary}"ENDIF/>
			    ELSE
			    <p:inputText id="field.toFaceletId" value="#{field.fieldValueExpression}" IF useTooltipForValidationMessagetitle="#{component.valid ? '' : facesContext.getMessageList(component.getClientId())[0].summary}"ENDIF field.disabledFlag field.requiredFlag>
				    field.attribute.type.createConverter
			    </p:inputText>
			    ENDIF
			    renderEndColumn(field)
		ENDIF
		IF !useTooltipForValidationMessage
			renderStartColumn(field)
			<p:message for="field.toFaceletId" display="icon" />
			renderEndColumn(field)
		ENDIF
'''	
	}
	
	def CharSequence renderStartColumn(EObject field) {'''
		IF !isRenderingInTabPage(field)
			<p:column>
		ENDIF	
	'''}
	
	def CharSequence renderEndColumn(EObject field) {'''
		IF !isRenderingInTabPage(field)
			</p:column>
		ENDIF	
	'''}
	
	def CharSequence requiredFlag(CustomizeableField field) {
		field.fieldFlags.requiredFlag
	}
	def CharSequence requriedFlag(IElementWithLayoutData widget) {
		widget.fieldFlags.requiredFlag
	}
	def CharSequence readonlyFlag(CustomizeableField field) {
		field.fieldFlags.readonlyFlag
	}
	def CharSequence readonlyFlag(IElementWithLayoutData widget) {
		widget.fieldFlags.readonlyFlag
	}
	def CharSequence disabledFlag(CustomizeableField field) {
		field.fieldFlags.disabledFlag
	}
	def CharSequence disabledFlag(IElementWithLayoutData widget) {
		widget.fieldFlags.disabledFlag
	}
	def CharSequence disabledFlag(EList<FieldFlag> flags) {
		val flag = flags.findFirst(f|f instanceof EnabledFlag)
		if (flag != null)
			'''disabled="#{!((flag as EnabledFlag).expression.compile)}"'''
	}
	def CharSequence readonlyFlag(EList<FieldFlag> flags) {
		val flag = flags.findFirst(f|f instanceof ReadOnlyFlag)
		if (flag != null)
			'''readonly="#{(flag as ReadOnlyFlag).expression.compile}"'''
	}
	def CharSequence requiredFlag(EList<FieldFlag> flags) {
		val flag = flags.findFirst(f|f instanceof RequiredFlag) 
		if (flag != null)
			'''required="#{(flag as RequiredFlag).expression.compile}"'''
	}
	
	def dispatch CharSequence createContents(Content content) {	'''
		FOR item : content.eContents
			beforeChildItem(false, false, 0, item)
			item.createContents
			afterChildItem(false, false, 0, item)
		ENDFOR
'''	
	}
	
	
	def dispatch CharSequence createContents(SetOfGuiElements s) { ''' 
		FOR item : s.elements
			val isCustomizeableField = isCustomizeableField(item)
			beforeChildItem((s.eContainer.eContainer as XmadslComposite).verticalPartitioning, !isCustomizeableField, 1, item)
			item.createContents
			afterChildItem((s.eContainer.eContainer as XmadslComposite).verticalPartitioning, !isCustomizeableField, 1, item)
		ENDFOR
''' 
	} 
	
	def dispatch CharSequence createContents(ReferencedXMAComposite composite) { 
		composite.createChildContents
	}
	
	def dispatch CharSequence createContents(XmadslComposite composite) { '''
		IF !composite.horizontalPartitioning && !composite.verticalPartitioning
			IF composite.compositeLabel != null
			<p:panel composite.uniqueIdentifierAtribute>
				<f:facet name="header">processResourceBundleProperty(composite, composite.compositeLabel)</f:facet>
				composite.createChildContents
			</p:panel>
			ELSE
			<h:panelGroup composite.uniqueIdentifierAtribute>
				composite.createChildContents
			</h:panelGroup>			
			ENDIF
		ELSEIF composite.horizontalPartitioning
			<p:panelGrid columns="getColumns(composite)" composite.uniqueIdentifierAtribute>
				IF composite.compositeLabel != null<f:facet name="header">processResourceBundleProperty(composite, composite.compositeLabel)</f:facet>ENDIF
				composite.createChildContents 
			</p:panelGrid>
		ELSEIF composite.verticalPartitioning
			<p:panelGrid composite.uniqueIdentifierAtribute>
				IF composite.compositeLabel != null<f:facet name="header">processResourceBundleProperty(composite, composite.compositeLabel)<p:row><p:column colspan="getColumns(composite)">composite.compositeLabel</p:column></p:row></f:facet>ENDIF
				composite.createChildContents 
			</p:panelGrid>
		ELSE
			composite.createChildContents				
		ENDIF
	'''	
	}	
	
	
	def dispatch CharSequence createContents(ComposeData data) { 
		/* Tabulator (ComposieData) should be handled by parent component --> */
	}
	def dispatch CharSequence createContents(AttachmentProperty property) { 
		/* should be handled by parent component */
	}
	def dispatch CharSequence createContents(EObject eObject) { '''
		    <!-- !!! No implementation for type 'eObject.eClass.name' !!! -->
	''' 
	}
	def dispatch CharSequence createContents(EventMappingList eObject) { 
		/* not handled here */
	}
	
	def CharSequence beforeChildContents(boolean vertical, int tabulators, CharSequence label) {

	}
	
	def CharSequence afterChildContents(boolean vertical, int tabulators) {

	}
	
	def boolean isRenderingInTabPage(EObject item) {
		return item.eContainer.eContainer instanceof TabPage;	
	}
	
	
	def String getItemName(EObject item) {
		var name = ""
		for (PropertyDescriptor pd : Introspector::getBeanInfo(item.getClass()).getPropertyDescriptors()) {
		  if (pd.getReadMethod() != null && "name".equals(pd.getName())) {
		    	name = pd.getReadMethod().invoke(item)?.toString()
		    }
		}
		
		if (name == "") {
			name = item.getClass().simpleName.toLowerCase + itemIndex
		}
		
		return name
	}
	
	def CharSequence beforeChildItem(boolean vertical, boolean generateColumn, int numberOfColumns, EObject item) {
		var content = ''''''
		if (isCustomizable(item)) {content = content + '''<ui:insert name="getItemName(item)">'''}
		itemIndex = itemIndex + 1
		if (!isRenderingInTabPage(item)) {
			if (vertical) {
				content = content + '''IF !(item.eContainer instanceof SetOfGuiElements)
					<p:row>
					ENDIF
						IF generateColumn
					<p:column colspan="numberOfColumns">
						ENDIF
				'''			
			}		
		}
		
		return content
	}
		
	def CharSequence afterChildItem(boolean vertical, boolean generateColumn, int numberOfColumns, EObject item) {
		var content = ''''''
		if (isCustomizable(item)) {content = content + '''</ui:insert>'''}
		if (!isRenderingInTabPage(item)) {
			if (vertical) {
			content = content + '''		
				IF generateColumn
					</p:column>
				ENDIF
				IF !(item.eContainer instanceof SetOfGuiElements)
					</p:row>
				ENDIF
			'''			
			}	
		} 
		
		return content
	}
	
	def dispatch CharSequence getCompositeLabel(ReferencedXMAComposite composite) {
		null
	}
	def dispatch CharSequence getCompositeLabel(XmadslComposite composite) {
		composite.labelText?.label
	}
	
	def dispatch CharSequence createChildContents(ReferencedXMAComposite composite) { 
		if (composite.content == null)
			return ''''''
		var tabulators = 1;
		if (composite.content.composeLayout != null && composite.content.composeLayout.tabulators != null )
			tabulators = composite.content.composeLayout.tabulators.size
		'''
		beforeChildContents(composite.verticalPartitioning, tabulators, null)
		FOR item : composite.content.eContents
			beforeChildItem(composite.verticalPartitioning, false, tabulators, item)
			item.createContents
			afterChildItem(composite.verticalPartitioning, false, tabulators, item)
		ENDFOR
		afterChildContents(composite.verticalPartitioning, tabulators)
		'''
	}
	def dispatch CharSequence createChildContents(XmadslComposite composite) {
		if (composite.content == null)
			return ''''''
			
		val numberOfColumns = getColumns(composite)
		var tabulators = 1;
		itemIndex = -1;
		if (composite.content.composeLayout != null && composite.content.composeLayout.tabulators != null )
			tabulators = composite.content.composeLayout.tabulators.size
		'''
		beforeChildContents(composite.verticalPartitioning, tabulators, composite.compositeLabel)
		FOR item : composite.content.eContents
			val generateColumn = !((isCustomizeableField(item)) || (item instanceof SetOfGuiElements))
			beforeChildItem(composite.verticalPartitioning, generateColumn, numberOfColumns, item)
			item.createContents
			afterChildItem(composite.verticalPartitioning, generateColumn, numberOfColumns, item)
		ENDFOR
		afterChildContents(composite.verticalPartitioning, tabulators)
		'''
	}
	
	/**
	 * Checks whether the item is a customizeable field
	 */
	def boolean isCustomizeableField(EObject item) {
		item instanceof CustomizeableField
	}
	
	def boolean hasEventMappings(Command command) {
		if (command.eContainer instanceof Component) {
			var commandComponent = command.eContainer as Component
			for (Page page : commandComponent.pages) {
				if (hasEventMappingForCommandInPage(command, page)) {
					return true
				}	
			}					
		} else if (command?.parentPage != null) {
			return hasEventMappingForCommandInPage(command, command.parentPage)
		}
		
		false
	}
	
	def boolean hasEventMappingForCommandInPage(Command command, Page page) {
		if (page != null) {
			if (page.events != null && page.events.mapping != null) {
				for (mapping : page.events?.mapping?.filter(typeof(GuiElementEventMapping))) {
					for (function : mapping.eventFunctions) {
						if (function.command != null && function.command == command) {
							return true
						}
					}
				}
			}
		}
		
		false	
	}
	
	def boolean hasEventMappings(IGuiElementWithEvent control, boolean client) {
		if (control.parentPage.events == null)
			return false;
		for (mapping : control.parentPage.events.mapping.filter(typeof(GuiElementEventMapping))) {
			for (function : mapping.eventFunctions)
				if (control == mapping.control) {
					if (function.command != null && function.command.clientOnly == client)
						return true
					if (function.predefinedCommand != null && function.command == null && client)
						return true
				}
		}
		false
	} 
	
	/**
	 * This function creates handlers of button events. It assumes that there is only one remotecommand.
	 * All uicommands associated with the button before a remotecommand are mapped to onclick event and
	 * all uicommands associated with the button after a remotecommand are mapped to oncomplete event.
	 */
	def CharSequence addFunctionCommands(IGuiElementWithEvent control, CharSequence listenerAttribute, CharSequence startAttribute) {
		val calls = new ArrayList<String>()
		if (control.parentPage.events == null)
			return ''
		for (mapping : control.parentPage.events?.mapping?.filter(typeof(GuiElementEventMapping))) {
			var remoteCommandAdded = false;
			var beforeCommands = new ArrayList<String>(); 
			var afterCommands = new ArrayList<String>();
			var remoteCommand = "";
			if (control == mapping.control) {
				for (function : mapping.eventFunctions) {
					if (function.command != null && function.command.clientOnly) {
						if (!remoteCommandAdded) {
							beforeCommands.add('''App.function.command.eContainer.widgetVar.function.command.name();''')
						} else {
							afterCommands.add('''App.function.command.eContainer.widgetVar.function.command.name();''');
						}
					} else if (function.command != null && !function.command.clientOnly) {
						remoteCommand = ''' listenerAttribute="#{control.parentPage.bean.function.command.name}"'''
						remoteCommandAdded = true
					} else if (function.predefinedCommand != null) {
						if (!remoteCommandAdded) {
							beforeCommands.add('''App.control.parentPage.widgetVar.function.predefinedCommand.literal();''')
						} else {
							afterCommands.add('''App.control.parentPage.widgetVar.function.predefinedCommand.literal();''');
						}
					}
				}
				
				
				if (beforeCommands.size > 0) {
					calls.add(''' startAttribute="''');
					calls.addAll(beforeCommands)
					if (!remoteCommandAdded)
						calls.add('''return false''');
					calls.add('''"''');
				}
				
				calls.add(remoteCommand)
				
				if (afterCommands.size > 0) {
					calls.add(''' oncomplete="''');
					for (commandName:afterCommands) {
						calls.addAll(afterCommands)									
					}
					calls.add('''"''');
				}
			}	
		}
		'''calls.join()'''
	} 
	
	
	def CharSequence columnValueExpression(TableColumn column) { 
		if (column.feature != null) {
			if (column.feature.feature != null && column.feature.feature.name != null)
				'''item.column.feature.feature.name'''
			else if (column.feature.feature != null && column.feature.feature.attribute != null && column.feature.feature.attribute.name != null) 
				'''item.column.feature.feature.attribute.name'''
		}
		 
	}
	def CharSequence fieldValueExpression(CustomizeableField field) { 
		'''IF (field.feature != null && field.feature.attributeHolder != null)field.object.eContainer.referencedBean.field.object.name.faceletName.field.feature.attributeHolder.name.faceletNameENDIF''' 
	} 

	
	
	def CharSequence toUiCommandPrototype(Command command) { '''
		IF command.clientOnly || !command.hasEventMappings
			command.name : function() {
				IF command.clientOnly
					IF command.implementation != null && command.implementation.statements?.size() > 0
						FOR statement : command.implementation.statements
							statement.createContents
		    			ENDFOR
		    		ELSE
					alert('uicommand "command.name" is not implemented. Please define it in custom .xhtml page ');
		    		ENDIF
	    		ELSE
   					command.widgetVar();
	    		ENDIF
	    },
	    ENDIF
	''' }
	
	def CharSequence toUiCommandPrototypes(EList<Command> commands) { '''
		FOR command : commands.filter(c|!c.name.equals("init"))
			command.toUiCommandPrototype
		ENDFOR
	'''
	}
	
	
	def dispatch CharSequence createContents(Statement stmt) {
		'''Other statement:stmt'''
	}
	def dispatch CharSequence createContents(InvokeStatement stmt) { '''
		FOR propertyJoin : stmt.propertyJoin
			IF propertyJoin.joinDirection == JoinDirection::IN
			propertyJoin.property.widgetVar.jq.val(
				IF propertyJoin.variable != null
				propertyJoin.variable.reference.widgetVar.propertyJoin.variable.reference.clientValue
				ELSEIF propertyJoin.stringLiteral != null
				'propertyJoin.stringLiteral'
				ELSEIF propertyJoin.integerLiteral != null
				'propertyJoin.integerLiteral'
				ENDIF
			); /* TODO: only valid for tables */
			ELSE
			// other direction: 
			ENDIF
		ENDFOR
		App.stmt.invokeAble.widgetVar.invoke();'''
	}
	
	def dispatch CharSequence clientValue(Table variable) {
		'''selection[0]'''
	} 
	def dispatch CharSequence clientValue(IReferenceableByStatementVariable variable) {
		''' no client value: variable '''
	} 
	
	/** TODO: Remove method after introducing i18n internationlization (texts are outsourced to language files!) */
	def CharSequence toEncodedHtml(String seq) {
		return seq.replaceAll("&", "&amp;");
	}

	
	
}