package org.openxma.dsl.generator.impl

import org.openxma.dsl.pom.model.ReferencedXMAComposite
import org.openxma.dsl.pom.model.XmadslComposite
import org.eclipse.emf.ecore.EObject
import org.openxma.dsl.pom.model.CustomizeableField
import org.openxma.dsl.pom.model.SetOfGuiElements
import org.openxma.dsl.pom.model.IElementWithLayoutData
import org.openxma.dsl.pom.model.TabPage
import org.openxma.dsl.pom.model.StyleSpecification
import org.openxma.dsl.pom.model.StylePropertyOther
import org.openxma.dsl.pom.model.StyleProperty

import static org.openxma.dsl.generator.helper.GeneratorConfigurationExtension.*
import org.openxma.dsl.pom.model.StylePropertyWidgetStyle
import org.openxma.dsl.pom.model.Button
class FaceletLayoutImpl {
	
	def dispatch boolean hasVerticalPartitioning(XmadslComposite composite) {
		return composite.verticalPartitioning
	}
	
	def dispatch boolean hasVerticalPartitioning(ReferencedXMAComposite composite) {
		return composite.verticalPartitioning
	}
	
			
	
	/**
	 * This methods returns the number of columns a PanelGrid should have.
	 * 
	 * In case of vertical panel, PanelGrid should have 3 columns if there is a customizeable field and 1 if there is no customizeable field
	 * In case of horizontal panel, PanelGrid should have 3 columns for each customizeable field plus one column of every other field
	 */
	def Integer getColumns(XmadslComposite composite) {		
		var columns = 0;
		
		if (composite.horizontalPartitioning) {
			if (composite.content != null && composite.content.eContents != null) {
				for (EObject item : composite.content.eContents) {
					columns = columns + numberOfColumnsForItem(item)
				}
			}
		} else if (composite.verticalPartitioning) {
			if (composite != null && composite.content != null && composite.content.eContents != null) {
				for (EObject item : composite.content.eContents) {
					var numberOfColumnsForItem = numberOfColumnsForItem(item)
					
					if (columns < numberOfColumnsForItem) {
						columns = numberOfColumnsForItem
					}
				}
			}
		}
		 
		return columns
	}
	
	def Integer numberOfColumnsForItem(EObject item) {
		switch item {
			CustomizeableField : if (useTooltipForValidationMessage) 2 else 3
			SetOfGuiElements : calculateNumberOfColumns(item)
			default: 1
		}
	}
	
	def Integer calculateNumberOfColumns(EObject item) {
		var columns = 0;
		for (EObject child : item.eContents) {
			columns = columns + numberOfColumnsForItem(child)
		}
		columns
	}
	
	/**
	 * This method returns the number of customizeable fields in the composite.
	 */
	def Integer numberOfSetofGuiElements(XmadslComposite composite) {
		if (composite != null && composite.content != null && composite.content.eContents != null) {
			composite.content.eContents.filter[item | 
				if (item.eContents != null) {
					item.eContents.filter(typeof(SetOfGuiElements)).size > 0 
				} else {
					false
				}
			].size
		} else {
			0
		}
	}
	
	/**
	 * This method returns the number of customizeable fields in the composite.
	 */
	def Integer numberOfCustomizableFields(XmadslComposite composite) {
		if (composite != null && composite.content != null && composite.content.eContents != null) {
			composite.content.eContents.filter[item | 
				if (item.eContents != null) {
					item.eContents.filter(typeof(CustomizeableField)).size > 0
				} else {
					false
				}
			].size
		} else {
			0
		}
	}
	
	def boolean isCustomizable(EObject object) {
		var customizable = false;
		if (object instanceof TabPage) {
			var styleProperty = (object as TabPage).styleProperty
			if (styleProperty != null) {
				customizable = styleProperty.styles?.exists[styleSpec | isCustomizableStyle((styleSpec as StyleSpecification))]
			}
		} else if (object instanceof IElementWithLayoutData) {
			customizable = (object as IElementWithLayoutData).layoutProperties?.filter(typeof(StyleProperty)).exists[layoutProperty | 
				(layoutProperty as StyleProperty).styles.exists[styleSpec | isCustomizableStyle((styleSpec as StyleSpecification))]
			]
		}
		
		return customizable
	} 
	
	def boolean isCustomizableStyle(StyleSpecification styleSpecification) {
		styleSpecification.styleProperty.filter(typeof(StylePropertyOther)).exists[property | (property as StylePropertyOther)?.customization?.equals("facelet")]
	}
	def CharSequence buttonStyle(Button b) {
		val style = b.stylePropertyWidgetStyle
		if (style != null)
			return '''styleClass="style"'''
		''''''
	}
	def CharSequence getStylePropertyWidgetProp(StyleSpecification styleSpec) {
		styleSpec.styleProperty.filter(typeof(StylePropertyWidgetStyle)).head?.predefinedWidgetStyle?.literal
	}
	def CharSequence getStylePropertyWidgetStyle(Button b) {
		for (layoutProp : b.layoutProperties?.filter(typeof(StyleProperty))) {
			for (styleSpec : (layoutProp as StyleProperty).styles.filter(typeof(StyleSpecification))) {
				val p = styleSpec.stylePropertyWidgetProp 
				if (p != null) {
					return p
				}
			}
		}
		null
	}
	
	
}