package org.openxma.dsl.generator.impl

import javax.inject.Inject
import org.openxma.dsl.core.model.IDocumentable
import org.openxma.dsl.dom.model.Attribute
import org.openxma.dsl.dom.model.ComplexType
import org.openxma.dsl.dom.model.DaoOperation
import org.openxma.dsl.dom.model.DataView
import org.openxma.dsl.dom.model.Entity
import org.openxma.dsl.dom.model.Parameter
import org.openxma.dsl.dom.model.QueryParameter
import org.openxma.dsl.dom.model.ValueObject
import org.openxma.dsl.generator.helper.EntityExtension
import org.openxma.dsl.generator.helper.JavaUtilExtension
import org.openxma.dsl.core.model.Type
import org.openxma.dsl.dom.model.Operation
import org.openxma.dsl.dom.model.DelegateOperation
import org.openxma.dsl.dom.model.AttributeGroup
import org.eclipse.emf.ecore.EObject
import org.eclipse.xtext.util.SimpleAttributeResolver
import org.openxma.dsl.core.model.Function
import org.openxma.dsl.core.model.SimpleType
import org.openxma.dsl.dom.model.DataBaseConstraint
import org.openxma.dsl.dom.model.DataBaseConstraintType
import org.openxma.dsl.dom.model.FeatureReference
import org.openxma.dsl.dom.model.QueryOperation
import org.openxma.dsl.dom.model.Dao

import static extension org.openxma.dsl.generator.helper.SimpleTypeExtension.*
import static extension org.openxma.dsl.common.extensions.EObjectPropertiesAdapter.*
import static extension org.openxma.dsl.common.Strings.*
import org.openxma.dsl.generator.FeatureExtensions

class Features {
	
	@Inject
	extension Names
	
	@Inject
	extension FeatureExtensions
	
	def getResolvedName(Attribute it) {
		if (getFQNProperty() != null) {
			getFQNProperty().toString().toFirstLower() + name.toFirstUpper()
		} else {
			name
		}
	}
	
	def String getFQNProperty(Attribute it) {
		getProperty("fqn")?.toString
	}
	
	
	def dispatch toType(Attribute it) {
		val qualifiedName = type.qualifiedName
		if (many) {
			if (eContainer instanceof Entity) {
				addImport("java.util.Set")+"<"+qualifiedName+">"	
			} else {
				addImport("java.util.List")+"<"+qualifiedName+">"	
			}
		} else {
			qualifiedName
		}
	}
	
	def dispatch toType(Operation it) {
		if (type == null) {
			return "void"
		} else {
			type.toType(many)
		}
	}
	
	def dispatch toType(DelegateOperation it) {
		if (operation.type == null) {
			return "void"
		} else if (view != null) {
			view.toType(operation.many)	  
		} else {
			operation.type.toType(operation.many)	  
		}
	}
	
	def dispatch toType(QueryParameter it) {
		attribute.dataType.toType(attribute.many)
	}
	
	def dispatch toType(Parameter it) {
		return type.toType(many)
	}

	def dispatch toType(DaoOperation it) {
		if (type == null) {
			return "void"
		} 
		return type.toType(many)
	}
	
	 def toType(Type it, boolean isCollectionType) {
		val qualifiedName = qualifiedName
		if (isCollectionType) {
			addImport("java.util.Collection")+"<"+qualifiedName+">"	
		} else {
			qualifiedName
		}
	}

	
	def getAddMethodName(Attribute it) {
		"add"+name.toFirstUpper
	}
	
	def getHasMethodName(Attribute it) {
		"has"+name.toFirstUpper
	}

	def getRemoveMethodName(Attribute it) {
		"remove"+name.toFirstUpper
	}
	
	def getRemoveAllMethodName(Attribute it) {
		"removeAll"+name.toFirstUpper
	}
	
	def dispatch String getAccessorMethodName(FeatureReference it) {
		attribute.accessorMethodName	
	}
	
	def dispatch String getAccessorMethodName(EObject it) {
		switch it {
			Attribute: "get" + it.name.toFirstUpper
			AttributeGroup: "get" + it.name.toFirstUpper
			default : {
				var name = SimpleAttributeResolver::NAME_RESOLVER.apply(it);
				if (booleanTyped) {
					"is"+name.toFirstUpper	
				} else {
					"get"+name.toFirstUpper
				}
			}
		}
	}
	
	def private boolean isBooleanTyped(EObject it) {
		switch it {
			org.openxma.dsl.dom.model.Property : it.type.booleanTyped
			Function : it.resultType.booleanTyped
			Attribute : it.dataType.booleanTyped
			SimpleType : "boolean".equalsIgnoreCase(it.getJavaType())
			default : false
		}
	}
	
	def getMethodName(DataBaseConstraint it) {
		switch (type) {
			case DataBaseConstraintType::NATURAL :
				"loadBy"+name
			case DataBaseConstraintType::UNIQUE :
				"loadBy"+name
			case DataBaseConstraintType::INDEX :
				"findBy"+name
			default : "NOT IMPLEMENTED"
		}
	}
	
	def getMutatorMethodName(Attribute it) {
		"set"+name.toFirstUpper
	}
	
	def boolean isIdOrVersionAttribute (Attribute it) {
		version || identifier
	}
	
	def generateAttributes(ComplexType it) {
		'''
		FOR attribute : attributes.filter([!derived])
		attribute.toField
		ENDFOR
		'''
	}
	
	def toField(Attribute it) {
		'''
		documentationIndented
		toFieldExtension
		protected toType nameIF type.dataType instanceof ValueObject = new IF manyaddImport("java.util.ArrayList")<type.qualifiedName>ELSEtype.qualifiedNameENDIF()ENDIF;
		'''
	}
	
	def dispatch generateProperties(DataView it) {
		'''
		FOR attribute : attributes
		
		IF !attribute.derived
		public IF attribute.manyList<attribute.type.qualifiedName>ELSEattribute.type.qualifiedNameENDIF attribute.getAccessorMethodName()() {
			return this.attribute.name;
		}
		
		public void attribute.getMutatorMethodName()(IF attribute.manyaddImport("java.util.List")<attribute.type.qualifiedName>ELSEattribute.type.qualifiedNameENDIF attribute.name) {
			this.attribute.name=attribute.name;
		}
		IF attribute.many
		
		public void attribute.getAddMethodName()(attribute.type.qualifiedName attribute.toVariableName) {
			if (attribute.toVariableName == null) {
				throw new IllegalArgumentException("parameter 'attribute.toVariableName' must not be null");
			}
			if (attribute.getAccessorMethodName()() == null) {
				this.attribute.toVariableName = new addImport("java.util.ArrayList")<attribute.type.qualifiedName>();
			}
			this.attribute.toVariableName.add(attribute.toVariableName);
		}
		ENDIF
		ELSE
		public abstract IF attribute.manyList<attribute.type.qualifiedName>ELSEattribute.type.qualifiedNameENDIF attribute.getAccessorMethodName()();
		IF !attribute.readOnly
		public abstract void attribute.getMutatorMethodName()(IF attribute.manyaddImport("java.util.List")<attribute.type.qualifiedName>ELSEattribute.type.qualifiedNameENDIF attribute.toVariableName);
		ENDIF
		ENDIF
		ENDFOR
		'''
	}
	
	def dispatch generateProperties(ComplexType it) {
		'''
		FOR attribute : attributes.filter([!derived])
		
		attribute.toGetterMethod
		IF !attribute.readOnly
		attribute.toSetterMethod
		ENDIF
		ENDFOR
		'''
	}
	
	def toGetterMethod(Attribute it) {
		'''
		IF many
			IF reference
			public Set<type.qualifiedName> accessorMethodName() {
				if (this.name == null) {
					this.name = new addImport("java.util.HashSet")<type.qualifiedName>();
				}
				return addImport("java.util.Collections").unmodifiableSet(this.name);
			}
			
			public boolean hasMethodName() {
				return name != null && !name.isEmpty();
			}
			
			public void addMethodName(type.qualifiedName name) {
				if (name.toFirstLower() == null) {
					throw new IllegalArgumentException("parameter 'name' must not be null");
				}
				if (!accessorMethodName().contains(name)) {
					this.name.add(name);
					IF oppositeReference!=null
					name.IF !oppositeReference.manyoppositeReference.mutatorMethodNameELSEoppositeReference.addMethodNameENDIF(this);
					ENDIF
				}
			}
			
			public void removeMethodName(type.qualifiedName name) {
				if (name == null) {
					throw new IllegalArgumentException("parameter 'name' must not be null");
				}
				if (accessorMethodName().contains(name)) {
					this.name.remove(name);
					IF oppositeReference!=null
					name.IF !oppositeReference.manyoppositeReference.mutatorMethodName(null)ELSEoppositeReference.removeMethodName(this)ENDIF;
					ENDIF
				}
			}
			
			public void removeAllMethodName() {
				if (this.name != null) {
					IF EntityExtension::isInverseSide(it) && !composition
					for (type.qualifiedName type.qualifiedName.toFirstLower() : new HashSet<type.qualifiedName>(this.name)) {
						getRemoveMethodName()(type.qualifiedName.toFirstLower());
					}
					ELSE
					this.name.clear();
					ENDIF
				}
			}
			ELSE
			public List<type.qualifiedName> getname.toFirstUpper() {
				return this.name.toFirstLower;
			}
			ENDIF
			
		ELSE
			public type.qualifiedName getname.toFirstUpper() {
				return this.name.toFirstLower;
			}
			
		ENDIF
		'''
	}
	
	def toSetterMethod(Attribute it) {
		'''
		IF many
			IF reference
			ELSE
			public void mutatorMethodName(List<type.qualifiedName> name) {
				this.name = name;
			}
			ENDIF
		ELSE
			IF reference
			public void mutatorMethodName(type.qualifiedName name) {
				if (this.name != name) {
					IF oppositeReference !=null && oppositeReference.many
					if (this.name != null) {
						this.name.oppositeReference.getRemoveMethodName()(this);
					}
					ENDIF
					this.name = name.toFirstLower();
					IF oppositeReference!=null
					if (name.toFirstLower() != null) {
						IF !oppositeReference.many
						name.toFirstLower().oppositeReference.getMutatorMethodName()(this);
						ELSE
						name.toFirstLower().oppositeReference.getAddMethodName()(this);
						ENDIF
					}
					ENDIF
				}
			}
			ELSE
			public void getMutatorMethodName()(type.qualifiedName name) {
				this.name = name;
			}
			ENDIF
		ENDIF
		'''
	}
	
	def generateToString(ComplexType complexType) {
		'''
		@Override
		public String toString() {
			StringBuilder toStringBuilder = new StringBuilder("complexType.name [");
			switch (complexType) {
				Entity case  complexType.superType != null: 
		 '''toStringBuilder.append(super.toString())IF !complexType.attributes.isEmpty.append(",")ENDIF;'''
				DataView case  complexType.superType != null: 
		 '''toStringBuilder.append(super.toString())IF !complexType.attributes.isEmpty.append(",")ENDIF;'''
			}
			FOR attribute : complexType.attributes.filter([!(dataType instanceof ComplexType)]) SEPARATOR ".append(\",\");\n" AFTER ";\n"toStringBuilder.append("attribute.name=").append(attribute.accessorMethodName())ENDFOR
			return toStringBuilder.append("]").toString();
		}
		'''
	}
	
	def String toSerialVersionUID(ComplexType it) {
		'''private static final long serialVersionUID = serialVersionUID();'''
	}
	
	def serialVersionUID(ComplexType it) {
		JavaUtilExtension::serialVersionUID(it)
	}	
	
	def toOperationSignature(Operation it, String visibility) {
	   '''
	   it.documentationIndented
	   IF visibility != nullpublic ENDIFtoType name(FOR paramameter : parameters SEPARATOR ","paramameter.methodParameterENDFOR)''' 
	}
	
	def toOperationSignature(DelegateOperation it, String visibility) {
	   '''
	   it.documentationIndented
	   IF visibility != nullpublic ENDIFtoType operation.name(FOR paramameter : (operation as Operation).parameters SEPARATOR ","paramameter.methodParameterENDFOR)
	   ''' 
	}

	def dispatch String methodParameterJavaDoc(Attribute it) {
		'''
		* @param getResolvedName()
		'''
	}
	
	def dispatch String methodParameterJavaDoc(Parameter it) {
		'''
		* @param name
		'''
	}
	
	def dispatch String methodParameterJavaDoc(QueryParameter it) {
		methodParameterJavaDoc(attribute)
	}
	
	def dispatch methodParameter(Attribute it) {
		'''toType resolvedName'''
	}
	
	def dispatch methodParameter(QueryParameter it) {
		'''toType attribute.name'''
	}
	
	def dispatch methodParameter(Parameter it) {
		'''toType name'''
	}
	
	def getDocumentation(IDocumentable it) {
		if (txtDescription != null) {
		'''
		/**
		 * txtDescription.replaceAll("\n", "\n* ")
		 */
		'''
		}
	}
	
	def getDocumentationLines(IDocumentable it) {
		if (txtDescription != null) {'''
		* txtDescription.replaceAll("\n", "\n* ")
		*
		'''
		}
	}
	
	def getDocumentationIndented(IDocumentable it) {
		if (txtDescription != null) {'''
		/** txtDescription.replaceAll("\n", "  ") */
		'''
		}
	}
	
	def getDocumentationLinesIndented(IDocumentable it) {
		if (txtDescription != null) {'''
		* txtDescription.replaceAll("\n", "\n	 * ")
		*
		'''
		}
	}
	
	def getQualifiedQueryName(QueryOperation it) {
        (eContainer as Dao).entity.name + "." + getQueryName
    }
    
    def getQualifiedQueryConstant(QueryOperation it) {
        (eContainer as Dao).name + "." + getQueryConstant
    }
	
	def getQueryConstant(QueryOperation it) {
        return queryName.toUnderScoreString.toUpperCase
    }
    
    def getQueryName(QueryOperation queryOperation) {
        val queryOperationsWithSameName = (queryOperation.eContainer as Dao).queryOperation.filter(
            [ queryOperation.name.equals(name)])
        if (queryOperationsWithSameName.size > 1) {
            queryOperation.name.toFirstUpper + queryOperationsWithSameName.toList().indexOf(queryOperation)
        } else {
            queryOperation.name.toFirstUpper
        }
    }
    
	def Type getType(QueryParameter it) {
		switch it {
			Parameter : it.type
			default : attribute.dataType
		}
	}
	
    def getName(QueryParameter it) {
		if (attribute != null) {
			return attribute.name
		} else {
			(it as Parameter).name	
		} 
	}
	
	def String getQualifiedKeyClassName(AttributeGroup it) {
		(eContainer as Entity).classPackageName + "." + keyClassName
	}
	
	def String getKeyClassName(AttributeGroup it) {
		name
	}

}