package org.openxma.dsl.generator.impl

import javax.inject.Inject
import org.eclipse.emf.ecore.resource.Resource
import org.eclipse.xtext.generator.IFileSystemAccess
import org.eclipse.xtext.generator.IGenerator
import org.openxma.dsl.core.model.DataTypeAndTypeParameter
import org.openxma.dsl.core.model.Model
import org.openxma.dsl.core.model.SimpleType
import org.openxma.dsl.dom.model.AggregateFunction
import org.openxma.dsl.dom.model.AliasedExpression
import org.openxma.dsl.dom.model.AltWhenClause
import org.openxma.dsl.dom.model.Attribute
import org.openxma.dsl.dom.model.BetweenExpression
import org.openxma.dsl.dom.model.BinaryExpression
import org.openxma.dsl.dom.model.BooleanLiteralValue
import org.openxma.dsl.dom.model.CallableStatement
import org.openxma.dsl.dom.model.CaseExpression
import org.openxma.dsl.dom.model.CastFunction
import org.openxma.dsl.dom.model.CollectionFunction
import org.openxma.dsl.dom.model.Column
import org.openxma.dsl.dom.model.Dao
import org.openxma.dsl.dom.model.DeleteStatement
import org.openxma.dsl.dom.model.EmptyLiteralValue
import org.openxma.dsl.dom.model.Expression
import org.openxma.dsl.dom.model.FromClass
import org.openxma.dsl.dom.model.FunctionCall
import org.openxma.dsl.dom.model.InClass
import org.openxma.dsl.dom.model.InCollection
import org.openxma.dsl.dom.model.InCollectionElements
import org.openxma.dsl.dom.model.InExpression
import org.openxma.dsl.dom.model.InsertStatement
import org.openxma.dsl.dom.model.IntegerLiteralValue
import org.openxma.dsl.dom.model.LikeExpression
import org.openxma.dsl.dom.model.LiteralValue
import org.openxma.dsl.dom.model.ManyToMany
import org.openxma.dsl.dom.model.ManyToOne
import org.openxma.dsl.dom.model.MemberOfExpression
import org.openxma.dsl.dom.model.NotExpression
import org.openxma.dsl.dom.model.NullLiteralValue
import org.openxma.dsl.dom.model.OneToMany
import org.openxma.dsl.dom.model.OneToOne
import org.openxma.dsl.dom.model.Parameter
import org.openxma.dsl.dom.model.ParenthesizedExpression
import org.openxma.dsl.dom.model.PropertyValue
import org.openxma.dsl.dom.model.QuantifiedExpression
import org.openxma.dsl.dom.model.QueryParameterValue
import org.openxma.dsl.dom.model.RealLiteralValue
import org.openxma.dsl.dom.model.SelectClass
import org.openxma.dsl.dom.model.SelectObject
import org.openxma.dsl.dom.model.SelectProperties
import org.openxma.dsl.dom.model.SelectStatement
import org.openxma.dsl.dom.model.StringLiteralValue
import org.openxma.dsl.dom.model.SubQuery
import org.openxma.dsl.dom.model.TrimFunction
import org.openxma.dsl.dom.model.UnaryExpression
import org.openxma.dsl.dom.model.UpdateStatement
import org.openxma.dsl.dom.model.WhenClause
import org.openxma.dsl.generator.lib.Index

import static org.openxma.dsl.generator.GeneratorConstants.*

import static extension org.openxma.dsl.generator.helper.EntityExtension.*
import static extension org.openxma.dsl.generator.helper.ValidatorExtension.*
import static extension org.openxma.dsl.generator.lib.IterableExtensions2.*
import static extension org.openxma.dsl.generator.lib.ValidatorExtensions.*

class HbmXmlFileGenerator implements IGenerator {
    @Inject
    extension Names
    @Inject
    extension Types
    @Inject
    extension Features

    override doGenerate(Resource resource, IFileSystemAccess fsa) {
        val model = resource.getEObject("/") as Model
        val daos = model.elements.filter(typeof(Dao))
        daos.filter([d1|!daos.exists(d2|d1.entity.superType == d2.entity)]).forEach(
            [   
                val hibernateMapping = toHibernateMapping
                fsa.generateFile(hbmXmlFileName, OUTLET_GENERATED_RESOURCES, hibernateMapping)
                fsa.generateFile(generatedHbmXmlFileName, OUTLET_GENERATED_RESOURCES, hibernateMapping)
                fsa.generateFile(fragmentHbmXmlFileName, OUTLET_RESOURCES, toHibernateFragmentMapping)
            ]
        )
    }
    
    def toHibernateFragmentMapping(Dao it) {
        '''
        <?xml version="1.0" encoding="UTF-8"?>
        <!DOCTYPE hibernate-mapping PUBLIC "-//Hibernate/Hibernate Mapping DTD//EN"
        "http://www.hibernate.org/dtd/hibernate-mapping-3.0.dtd">
        <hibernate-mapping default-access="field" default-lazy="true">
            <!-- The content of this file will be merged into: hbmXmlFileName-->
            <!-- In order for merging to be successful, the merged elements have to placed into the right xml hierarchy position.-->
            <class name="entity.getQualifiedClassName()"
                   table="tableName.toUpperCase()" discriminator-value="qualifier">
                <!-- place for customised properties -->
            </class>
            <!-- place for customised queries -->
        </hibernate-mapping>
        '''
    }
   
    
    def toHibernateMapping(Dao it) {
        '''
        <?xml version="1.0" encoding="UTF-8"?>
        <!DOCTYPE hibernate-mapping PUBLIC "-//Hibernate/Hibernate Mapping DTD//EN"
                "http://hibernate.sourceforge.net/hibernate-mapping-3.0.dtd">
        <hibernate-mapping default-access="field" default-lazy="true">
            <class name="entity.getQualifiedClassName()"
                   table="tableName.toUpperCase()" discriminator-value="qualifier">
                IF entity.txtDescription!=null
                <comment><![CDATA[entity.txtDescription]]></comment>
                ENDIF
                IF primaryKey!=null
                <id name="primaryKeyColumn.attribute.name" access="property">
                    primaryKeyColumn.toColumnMapping
                    primaryKeyColumn.toTypeMapping
                    <generator class="assigned" />
                </id>
                ELSE
                <composite-id class="entity.key.qualifiedKeyClassName" mapped="true">
                FOR naturalKeyColumn : naturalKeyColumns
                    <key-property name="naturalKeyColumn.attribute.name">
                        naturalKeyColumn.toColumnMapping
                        naturalKeyColumn.toTypeMapping
                    </key-property>
                ENDFOR
                </composite-id>
                ENDIF
                IF !getSubClassDaos.empty
                <discriminator column="discriminator" type="string"/>
                ENDIF
                IF versionColumn!=null
                val String versionType = versionColumn.attribute.type.qualifiedName
                <version name="versionColumn.attribute.name" access="field" unsaved-value="null" IF versionType.endsWith("Date") || versionType.endsWith("Timestamp")type="timestamp"ELSEIF versionType.endsWith("Long")type="long"ENDIF>
                    versionColumn.toColumnMapping
                </version>
                ENDIF
                val propertyColumns = columns.reject(e|e.attribute.identifier || e.attribute.version || it.manyToOneAssociations.map(m|m.columnName).contains(e.columnName))
                IF primaryKey!=null
                FOR property : propertyColumns
                property.toPropertyMapping
                ENDFOR
                ELSE
                FOR property : propertyColumns.reject(e|it.naturalKeyColumns.contains(e))
                property.toPropertyMapping
                ENDFOR
                ENDIF
                FOR relation : oneToManyAssociations
                relation.toRelationMapping
                ENDFOR
                FOR relation : manyToOneAssociations
                relation.toRelationMapping
                ENDFOR
                FOR relation : oneToOneAssociations
                relation.toRelationMapping
                ENDFOR
                FOR relation : manyToManyAssociations
                relation.toRelationMapping
                ENDFOR
                FOR subClassDao : getSubClassDaos
                <subclass name="subClassDao.entity.getQualifiedClassName()" discriminator-value="subClassDao.qualifier">
                    FOR property : subClassDao.columns.filter(e|subClassDao.entity.attributes.exists(sca|sca.name==e.attribute.name))
                    property.toPropertyMapping
                    ENDFOR
                    FOR relation : subClassDao.oneToManyAssociations
                    relation.toRelationMapping
                    ENDFOR
                    FOR relation : subClassDao.manyToOneAssociations
                    relation.toRelationMapping
                    ENDFOR
                    FOR relation : subClassDao.oneToOneAssociations
                    relation.toRelationMapping
                    ENDFOR
                    FOR relation : subClassDao.manyToManyAssociations
                    relation.toRelationMapping
                    ENDFOR
                </subclass>
                ENDFOR
            </class>
            FOR operation : operations.filter(e|e.expression!=null)
            <query name="entity.name.operation.name.toFirstUpper()"><![CDATA[ operation.expression ]]></query>
            ENDFOR
            FOR queryOperation : queryOperation.reject(e|e.statement instanceof CallableStatement)
            <query name="queryOperation.getQualifiedQueryName()">
            <![CDATA[
queryOperation.statement.toQlString
            ]]>
            </query>
            ENDFOR
            FOR subClassDao : getSubClassDaos
            subClassDao.toNamedQueryMapping
            ENDFOR
        </hibernate-mapping>
        '''
    }
    
    def toNamedQueryMapping(Dao it) {
        '''
        FOR operation : operations.filter(e|e.expression!=null)
        <query name="entity.name.operation.name.toFirstUpper()"><![CDATA[ operation.expression ]]></query>
        ENDFOR
        FOR queryOperation : queryOperation.reject(e|e.statement instanceof CallableStatement)
        <query name="queryOperation.getQualifiedQueryName()">
        <![CDATA[
        queryOperation.statement.toQlString
        ]]>
        </query>
        ENDFOR
        '''
    }
   
    def toColumnMapping(Column it) {
        '''
        <column name="columnName"IF attribute.required not-null="true"ENDIFIF columnType!=null sql-type="columnType.qualifiedName"ENDIFIF attribute.type.getMaxLength()!=null length="attribute.type.getMaxLength()"ENDIFIF attribute.type.getPrecision()!=null precision="attribute.type.getPrecision()"ENDIFIF attribute.type.getScale()!=null scale="attribute.type.getScale()"ENDIFIF comment.nullOrEmpty/ENDIF>
            IF !comment.nullOrEmpty
            <comment><![CDATA[comment]]></comment>
            ENDIF
        IF !comment.nullOrEmpty
        </column>
        ENDIF
        '''   
    }
    
    def toTypeMapping(Column it) {
        if (userType == null) {
            return ""
        }
        '''
        IF userType.parameterValues.empty
        <type name="userType.qualifiedClassName"/>
        ELSE
        <type name="userType.qualifiedClassName">
        val index = new Index
        FOR parameterValue : userType.parameterValues
            <param name="(userType.dataType as SimpleType).typeParameter.get(index.counter0).name">parameterValue.getValueAsString()</param>
            index.increment
        ENDFOR
        </type>
        ENDIF
        '''
    }
    
    def toPropertyMapping(Column it) {
        '''
        IF !columns.isEmpty
        <component name="attribute.name" class="attribute.type.qualifiedClassName" access="field">
        FOR column : columns
            <property name="column.attribute.name">
                column.toColumnMapping
                column.toTypeMapping
            </property>
        ENDFOR
        </component>
        ELSE
        <property name="attribute.name">
            toColumnMapping
            toTypeMapping
        </property>
        ENDIF
        '''  
    }
    
    def toRelationMapping(OneToMany it) {
        '''
        <set name="attribute.name" inverse="attribute.opposite!=null" cascade="allIF attribute.composition,delete-orphanENDIF"IF attribute.sortOrder!=null order-by="FOR order : attribute.sortOrder.attributes SEPARATOR ","getColumnName((eContainer as Dao).entity,order.attribute) if (order.asc) {"ASC"} else {"DESC"}ENDFOR"ENDIFIF !attribute.composition optimistic-lock="false"ENDIF>
            IF !columns.isEmpty
            <key>
            FOR foreignKeyColumn : columns
                <column name="foreignKeyColumn.columnName"/>
            ENDFOR
            </key>
            ELSE
            <key column="columnName"/>
            ENDIF
            <one-to-many class="attribute.type.qualifiedClassName"/>
        </set>
        '''
    }
    
    def toRelationMapping(ManyToMany it) {
        '''
        <set name="attribute.name" inverse="inverse" table="tableName" cascade="save-update">
            <key column="columnName"/>
            <many-to-many class="attribute.type.qualifiedClassName"  column="getJoinColumnName()"/>
        </set>
        '''    
    }
    
    def toRelationMapping(ManyToOne it) {
        '''
        <many-to-one name="attribute.name" class="attribute.type.qualifiedClassName"IF derived insert="false" update="false"ENDIF>
            IF !columns.isEmpty
            FOR foreignKeyColumn : columns
            <column name="foreignKeyColumn.columnName"/>
            ENDFOR
            ELSE
            <column name="columnName"IF sqlType !=null sql-type="sqlType.ddl"ENDIFIF attribute.required not-null="true"ENDIF/>
            ENDIF
        </many-to-one>
        '''    
    }
    
    def toRelationMapping(OneToOne it) {
        '''
        <one-to-one name="attribute.name" class="attribute.type.qualifiedClassName" IF attribute.composition || attribute.opposite.requiredcascade="all"ENDIF property-ref="attribute.opposite.name"/>
        '''    
    }
    
    
    def getSubClassDaos(Dao it) {
        entity.model.eAllContents.filter(typeof(Dao)).filter(e|e.entity.superType==entity).toList.sortBy(e|e.name)
    }
    
    
    def dispatch toQlString(DeleteStatement it) {
        '''
        delete it.entity.className as it.name IF it.where!=nullwhere it.where.toExpressionString ENDIF
        '''         
     }
     
     def dispatch toQlString(InsertStatement it) {
        '''
        insert into it.entity.getClassName() (FOR expression : it.expression SEPARATOR ','expression.toExpressionStringENDFOR)  it.selectStatement.toQlString
        '''        
     }
     
     def dispatch toQlString(UpdateStatement it) {
        '''
        updateIF versioned versionedENDIF entity.className as name
        set FOR propertyAssignment : assignment SEPARATOR ','propertyAssignment.property.toExpressionString = propertyAssignment.expression.toExpressionStringENDFOR
        IF where!=null
        where where.toExpressionString
        ENDIF
        '''       
     }
     
     def dispatch toQlString(SelectStatement it) {
        '''toSelectString'''
     }
     
     def dispatch toQlString(SelectProperties it) {
        '''select IF distinctdistinct ENDIFFOR property : properties SEPARATOR ','property.toExpressionStringENDFOR toSelectString'''
     }
     
     def dispatch toQlString(SelectClass it) {
        '''select new class_(FOR argument : arguments SEPARATOR ','argument.toExpressionStringENDFOR)
        toSelectString'''
     }
     
     def dispatch toQlString(SelectObject it) {
        '''select object(name) toSelectString'''
     }
    
    def toSelectString(SelectStatement it) {
		'''
        IF !from.isEmpty
        from FOR from : from SEPARATOR ','from.toFromRangeStringENDFORENDIFFOR join : join
        IF join.type!=null join.typeENDIF join IF join.fetchfetch ENDIFjoin.entity.name.join.reference.name as join.name IF join.propertyFetchfetch all propertiesENDIFIF join.expression!=null with join.expression.toExpressionStringENDIFENDFORIF where != null
        where where.toExpressionStringENDIFIF !groupBy.isEmpty
        group by FOR groupBy : groupBy SEPARATOR ','groupBy.toExpressionStringENDFORIF having!=null having having.toExpressionString ENDIFENDIFIF !orderBy.isEmpty
        order by FOR orderBy : orderBy SEPARATOR ','orderBy.expression.toExpressionString orderBy.sortOrderENDFORENDIF'''
    }
    
    def dispatch toFromRangeString(InClass it) {
        '''name in class class_'''
    }
    
    def dispatch toFromRangeString(InCollection it) {
        '''in (path) alias'''
    }
    
    def dispatch toFromRangeString(InCollectionElements it) {
        '''name in elements (reference)'''
    }
    
    def dispatch toFromRangeString(FromClass it) {
        '''entity.getClassName() as nameIF popertyFetch fetch all propertiesENDIF'''
    }
    
    def dispatch toExpressionString(Expression it) {
    }

    def dispatch toExpressionString(AliasedExpression it) {
        '''expression.toExpressionString IF name!=null as name ENDIF'''
    }
    
    def dispatch toExpressionString(BinaryExpression it) {
        '''left.toExpressionString operator right.toExpressionString'''
    }
    
    def dispatch toExpressionString(UnaryExpression it) {
        '''operatorexpression.toExpressionString'''
    }
    
    def dispatch toExpressionString(NotExpression it) {
        ''' not expression.toExpressionString'''
    }
    
    def dispatch toExpressionString(InExpression it) {
        '''expression.toExpressionStringIF not notENDIF operator in.toExpressionString'''
    }
    
    def dispatch toExpressionString(BetweenExpression it) {
        '''expression.toExpressionStringIF not notENDIF operator left.toExpressionString and right.toExpressionString'''
    }
    
    def dispatch toExpressionString(LikeExpression it) {
        '''expression.toExpressionStringIF not notENDIF operator like.toExpressionStringIF escape!=null escape escape.toExpressionStringENDIF'''
    }
    
    def dispatch toExpressionString(MemberOfExpression it) {
        '''expression.toExpressionStringIF not notENDIF operator of memberOf'''
    }
    
    def dispatch toExpressionString(PropertyValue it) {
        '''nameFOR segment : segments.segmentENDFOR'''
    }
    
    def dispatch toExpressionString(FunctionCall it) {
        '''function(FOR argument : arguments SEPARATOR ','argument.toExpressionStringENDFOR)'''
    }
    
    def dispatch toExpressionString(TrimFunction it) {
        '''trim(IF mode!=nullmode ENDIFIF character!=nullcharacter from ENDIFfrom.toExpressionString)'''
    }
    
    def dispatch toExpressionString(CastFunction it) {
    	'''
    	
    	IF "cast".equalsIgnoreCase(function)
    	function(from.toExpressionString as name)
    	ELSE
    	function(name from from.toExpressionString)
    	ENDIF
    	'''
	}
	
    def dispatch toExpressionString(AggregateFunction it) {
        if (aggregateExpression!=null) {
            '''function(aggregateExpression.toExpressionString)'''    
        } else {
            '''function(IF all*ELSEIF distinctdistinct ENDIFIF from!=nullfromELSEIF collection!=nullcollection.toExpressionStringENDIF)'''
        }
    }
    
    def dispatch toExpressionString(QueryParameterValue it) {
        ''':nameIF attribute!=null_attribute.nameENDIF'''
    }
    
    def dispatch toExpressionString(QuantifiedExpression it) {
        '''quantifier expression.toExpressionString name'''
    }
    
    def dispatch toExpressionString(CaseExpression it) {
		'''
		
		IF expression == null
		case FOR whenClause : whenClausewhenClause.toExpressionStringENDFOR IF null!=elseExpression else elseExpression.toExpressionString ENDIF end
		ELSE
		case expression.toExpressionString FOR expression : altWhenClauseexpression.toExpressionStringENDFORIF elseExpression!=null else elseExpression.toExpressionString ENDIF end
		ENDIF
		'''
    }
    
    def dispatch toExpressionString(WhenClause it) {
        ''' when whenExpression.toExpressionString then thenExpression.toExpressionString'''   
    }
    
    def dispatch toExpressionString(AltWhenClause it) {
        ''' when whenExpression.toExpressionString then thenExpression.toExpressionString'''
    }
    
    def dispatch toExpressionString(CollectionFunction it) {
        '''function property.toExpressionString'''
    }
    
    def dispatch toExpressionString(SubQuery it) {
        '''
        (
        FOR statement : queries SEPARATOR 'union'statement.toQlStringENDFOR)'''
    }
    
    def dispatch toExpressionString(ParenthesizedExpression it) {
        '''(FOR expression : expressions SEPARATOR ','expression.toExpressionStringENDFOR)'''
    }
    
    def dispatch toExpressionString(StringLiteralValue it) {
        "'" + value + "'"
    }
    
    def dispatch toExpressionString(IntegerLiteralValue it) {
        value
    }
    
    def dispatch toExpressionString(LiteralValue it) {
        switch it {
            StringLiteralValue :  "'" + it.value + "'"   
            BooleanLiteralValue case it.isTrue : "true" 
            BooleanLiteralValue case !it.isTrue : "false" 
            EmptyLiteralValue : "empty" 
            NullLiteralValue : "null"
            IntegerLiteralValue : it.value 
            RealLiteralValue : it.value 
        }
    }
    
    def getName(QueryParameterValue it) {
        val type = parameter
        switch type {
            Parameter : type.name
            Attribute : type.name
        }
    }
    
    def private String getQualifiedClassName(DataTypeAndTypeParameter it) {
		getQualifiedClassName(dataType)
	}
    
 }
