package org.openxma.dsl.generator.impl

import com.google.inject.Inject
import org.eclipse.emf.common.util.EList
import org.eclipse.emf.ecore.EObject
import org.openxma.dsl.core.model.DataTypeAndTypeParameter
import org.openxma.dsl.core.model.EnumType
import org.openxma.dsl.core.model.SimpleType
import org.openxma.dsl.core.model.impl.StringParameterValueImpl
import org.openxma.dsl.dom.model.Service
import org.openxma.dsl.pom.model.CheckBox
import org.openxma.dsl.pom.model.Content
import org.openxma.dsl.pom.model.CustomizeableField
import org.openxma.dsl.pom.model.DataObjectVariable
import org.openxma.dsl.pom.model.ObjectProperty
import org.openxma.dsl.pom.model.ReferencedXMAComposite
import org.openxma.dsl.pom.model.SetOfGuiElements
import org.openxma.dsl.pom.model.TabFolder
import org.openxma.dsl.pom.model.TabPage
import org.openxma.dsl.pom.model.Table
import org.openxma.dsl.pom.model.Tree
import org.openxma.dsl.pom.model.XmadslComposite
import org.openxma.dsl.pom.model.XmadslPage
import org.openxma.dsl.pom.model.Text
import org.openxma.dsl.pom.model.Combo
import org.openxma.dsl.pom.model.ListBox
import org.openxma.dsl.pom.model.TableCombo
import org.openxma.dsl.core.model.StatusFlag

class JsfDataImpl implements JsfData {
	@Inject
	extension Names names
	@Inject
	extension JsfNames jsfNames
	@Inject
	extension Types type
	
	override CharSequence toDataObjectVars(EList<DataObjectVariable> variables) { '''
		FOR dataObject : variables
			IF dataObject.isCollection
	protected dataObject.addImport("java.util.Collection")<dataObject.type.pomType.qualifiedName> dataObject.name;
			ELSE
	protected dataObject.type.pomType.qualifiedName dataObject.name;
			ENDIF
		ENDFOR
	'''
	}
	
	override CharSequence toObjectPropertyVars(EList<ObjectProperty> properties) { '''
	FOR objProperty : properties
	protected objProperty.type.qualifiedName objProperty.name;
	ENDFOR
	'''
	}
	
	override CharSequence toInjectedDependencies(EList dependencies) { '''
	FOR service : dependencies.filter(typeof(Service))
		@service.addImport("org.springframework.beans.factory.annotation.Autowired")
	protected service.qualifiedName service.name.toFirstLower();
	ENDFOR
	'''
	}

	def dispatch CharSequence toGetterAndSetter(ObjectProperty objProperty){
		toGetterAndSetter(objProperty.type.qualifiedName, objProperty.name)
	}
	
	def CharSequence toGetterAndSetter(CharSequence fullyQualifiedName, String propertyName){ '''
	    public fullyQualifiedName getpropertyName.toFirstUpper() {
	        return propertyName;
	    }
	    public void setpropertyName.toFirstUpper(fullyQualifiedName propertyName) {
	        this.propertyName = propertyName;
	    }
	'''
	}
	
	def dispatch CharSequence toGetterAndSetter(DataObjectVariable dataObject){
		if (dataObject.isCollection) 
			toGetterAndSetter(dataObject.addImport("java.util.Collection"), dataObject.name)
		else
			toGetterAndSetter(dataObject.type.pomType.qualifiedName, dataObject.name)
	}
	def dispatch CharSequence toGetterAndSetter(StatusFlag flag){
		toGetterAndSetter("Boolean", flag.name)
	}
	
	def dispatch CharSequence getModelPropertyName(Table table) {
		'''table.nameTableModel'''
	}
	def dispatch CharSequence getRootNodePropertyName(Tree tree) {
		'''tree.nameTreeRootNode'''
	}
	def dispatch CharSequence getSelectedNodePropertyName(Tree tree) {
		'''tree.nameTreeSelectedNode'''
	}
	def dispatch CharSequence getModelPropertyNameFirstUpper(Table table) {
		'''table.name.toFirstUpperTableModel'''
	}
	def dispatch CharSequence getRootNodePropertyNameFirstUpper(Tree tree) {
		'''tree.name.toFirstUpperTreeRootNode'''
	}
	def dispatch CharSequence getSelectedNodePropertyNameFirstUpper(Tree tree) {
		'''tree.name.toFirstUpperTreeSelectedNode'''
	}
	def dispatch CharSequence getSelectItemsPropertyName(EObject eobject) {
		throw new RuntimeException("getSelectItemsPropertyName: Not supported class " + eobject.getClass().name)
	}
	def dispatch CharSequence getSelectItemsPropertyName(CustomizeableField field) {
		'''field.attribute.name1SelectItems'''
	}
	def dispatch CharSequence getSelectItemsPropertyName(Combo combo) {
		'''combo.name1SelectItems'''
	}
	def dispatch CharSequence getSelectItemsPropertyName(ListBox listbox) {
		'''listbox.name1SelectItems'''
	}
	def dispatch CharSequence getSelectItemsPropertyName(TableCombo combo) {
		'''combo.name1SelectItems'''
	}
	
	def dispatch CharSequence toJsfDataModel(Tree it) { '''
		protected addImport("org.primefaces.model.TreeNode") rootNodePropertyName;
		protected addImport("org.primefaces.model.TreeNode") selectedNodePropertyName;
		
		toGetterAndSetter(addImport("org.primefaces.model.TreeNode"),rootNodePropertyName.toString)
		toGetterAndSetter(addImport("org.primefaces.model.TreeNode"),selectedNodePropertyName.toString)
	'''
	}
	def dispatch CharSequence toJsfDataModel(Table it) { '''
		protected addImport("org.openxma.dsl.platform.jsf.model.MapDataModel")<key.attribute.dataType.qualifiedName,object.type.pomType.qualifiedName> modelPropertyName = new MapDataModel<key.attribute.dataType.qualifiedName,object.type.pomType.qualifiedName>() {
			public key.attribute.dataType.qualifiedName getRowKey(object.type.pomType.qualifiedName row) {
				return row.getkey.attribute.name.toFirstUpper();
			}
		};
		
		toGetterAndSetter(addImport("org.openxma.dsl.platform.jsf.model.MapDataModel"),modelPropertyName.toString)
	'''
	}
	def dispatch CharSequence toJsfDataModel(CustomizeableField it) {
		 '''
		IF (attribute.type.dataType instanceof EnumType)
		protected addImport("java.util.List")<addImport("javax.faces.model.SelectItem")> selectItemsPropertyName;
		
		toGetterAndSetter(addImport("java.util.List"), selectItemsPropertyName.toString)
		ELSEIF (attribute.type.validatorReferenceDefinition.i18nValidator)
		protected addImport("java.util.List")<addImport("javax.faces.model.SelectItem")> selectItemsPropertyName;
		
		toGetterAndSetter(addImport("java.util.List"), selectItemsPropertyName.toString)
		ENDIF
	'''
	}
	def dispatch CharSequence toJsfDataModel(CheckBox checkBox) { '''
		protected Boolean checkBox.getBackingBeanVar = false;
		
		toGetterAndSetter("Boolean", checkBox.getBackingBeanVar)
	'''
	}
	def dispatch CharSequence toJsfDataModel(Text text) { '''
		protected String text.getBackingBeanVar = "";
		
		toGetterAndSetter("String", text.getBackingBeanVar)
	'''
	}
	def dispatch CharSequence toJsfDataModel(Combo it) { '''
		protected addImport("java.util.List")<addImport("javax.faces.model.SelectItem")> selectItemsPropertyName;
				
		toGetterAndSetter(addImport("java.util.List"), selectItemsPropertyName.toString)
		
		protected Object getBackingBeanVar;
		
		toGetterAndSetter("Object", getBackingBeanVar)
	'''
	}
	
	def dispatch CharSequence toJsfDataModel(TableCombo it) { '''
		protected Object getBackingBeanVar();
		
		toGetterAndSetter("Object", getBackingBeanVar())
	'''
	}
	
	def dispatch CharSequence toJsfDataModel(ListBox it) { '''
		protected addImport("java.util.List")<addImport("javax.faces.model.SelectItem")> selectItemsPropertyName;
				
		toGetterAndSetter(addImport("java.util.List"), selectItemsPropertyName.toString)
		
		protected Object getBackingBeanVar();
		
		toGetterAndSetter("Object", getBackingBeanVar())
	'''
	}
	
	def dispatch CharSequence toJsfDataModel(EObject o) {
		/* nothing  */  
	}
	def dispatch CharSequence toJsfDataModel(XmadslPage page) { 
		page.content?.toJsfDataModel		
	} 
	def dispatch CharSequence toJsfDataModel(XmadslComposite composite) { 
		composite.content?.toJsfDataModel		
	} 
	def dispatch CharSequence toJsfDataModel(ReferencedXMAComposite composite) { 
		composite.content.toJsfDataModel		
	} 
	def dispatch CharSequence toJsfDataModel(SetOfGuiElements composite) {'''
		FOR item : composite.eContents
			item.toJsfDataModel
		ENDFOR
		'''
	} 
	def dispatch CharSequence toJsfDataModel(TabFolder tabFolder) { '''
		FOR item : tabFolder.eContents
			item.toJsfDataModel
		ENDFOR
		'''
	} 
	def dispatch CharSequence toJsfDataModel(TabPage page) { ''' 
		FOR item : page.eContents
			item.toJsfDataModel
		ENDFOR
		'''
	} 
	def dispatch CharSequence toJsfDataModel(Content content) { '''
		FOR item : content.eContents
			item.toJsfDataModel
		ENDFOR
	'''
	} 
	def dispatch CharSequence toInitCode(EObject it) {
		/* nothing */
	}
	def dispatch CharSequence toInitCode(CustomizeableField it) {
		 '''
		IF (attribute.type.dataType instanceof EnumType)
		ELSEIF (attribute.type.validatorReferenceDefinition.i18nValidator)
		selectItemsPropertyName = addImport("org.openxma.dsl.platform.jsf.util.DomainValuesUtil").getSelectItems(addImport("at.spardat.xma.datasource.Domain").getInstance("attribute.type.domainSpec", xmaSession).getDomRows(false));
		ENDIF
	'''
	}
	
	def CharSequence getDomainSpec(DataTypeAndTypeParameter type) {
		val simpleType = type.dataType as SimpleType
		val definition = simpleType.typeDefinition.definition
		'''TODO:testspec (definition.parameterValues.get(0) as StringParameterValueImpl).stringValue(definition.parameterValues.get(1) as StringParameterValueImpl).stringValue'''
	}
	
	def dispatch CharSequence toInitCode(XmadslPage page) { 
		page.content?.toInitCode		
	} 
	def dispatch CharSequence toInitCode(XmadslComposite composite) { 
		composite.content?.toInitCode		
	} 
	def dispatch CharSequence toInitCode(ReferencedXMAComposite composite) { 
		composite.content.toInitCode		
	} 
	def dispatch CharSequence toInitCode(SetOfGuiElements composite) {'''
		FOR item : composite.eContents
			item.toInitCode
		ENDFOR
		'''
	} 
	def dispatch CharSequence toInitCode(TabFolder tabFolder) { '''
		FOR item : tabFolder.eContents
			item.toInitCode
		ENDFOR
		'''
	} 
	def dispatch CharSequence toInitCode(TabPage page) { ''' 
		FOR item : page.eContents
			item.toInitCode
		ENDFOR
		'''
	} 
	def dispatch CharSequence toInitCode(Content content) { '''
		FOR item : content.eContents
			item.toInitCode
		ENDFOR
	'''
	} 
}