package org.openxma.dsl.generator.impl

import javax.inject.Inject
import org.openxma.dsl.generator.lib.ImportAdapter
import org.eclipse.emf.ecore.resource.Resource
import org.eclipse.emf.mwe.core.monitor.ProgressMonitor
import org.eclipse.xtext.generator.IFileSystemAccess
import org.eclipse.xtext.generator.IGenerator
import org.openxma.dsl.core.model.Model
import org.openxma.dsl.generator.Generator
import org.openxma.dsl.generator.GeneratorConfiguration
import org.openxma.dsl.generator.GeneratorConstants
import org.openxma.dsl.pom.model.Component
import org.openxma.dsl.pom.model.XmadslPage
import org.apache.commons.logging.Log
import java.util.List
import com.google.common.collect.Lists
import org.eclipse.emf.ecore.resource.ResourceSet
import org.eclipse.emf.common.notify.Notifier
import org.openxma.dsl.generator.IFileSystemAccessProvider
import java.util.HashMap
import java.util.Map
import java.lang.CharSequence
import java.util.ArrayList
import java.io.StringWriter
import java.util.LinkedHashMap
import org.openxma.dsl.generator.helper.GeneratorConfigurationExtension

import static org.apache.commons.logging.LogFactory.*

import static extension org.openxma.dsl.common.extensions.EObjectPropertiesAdapter.*

class JsfGenerator implements Generator, IGenerator { 
	val Log logger = getLog(getClass().getName());
	GeneratorConfiguration generatorConfiguration
	@Inject
	extension Facelet facelet
	@Inject
	extension BackingBean backingBean
    @Inject
    extension Files files
    @Inject 
    IFileSystemAccessProvider fileSystemAccessProvider;
    @Inject
    extension AutomationTestCaseGenerator automationTestCaseGenerator;
	
	override init(GeneratorConfiguration generatorConfiguration) {
		this.generatorConfiguration = generatorConfiguration
	}
	
	override generate(Resource resource, ProgressMonitor progressMonitor) {
		resource.resourceSet.modify
		val fsa = new IGeneratorDelegate(fileSystemAccessProvider.get(generatorConfiguration), resource.resourceSet)
		doGenerate(resource,fsa)
	}

	override doGenerate(Resource input, IFileSystemAccess fsa) {
		val model = input.getEObject("/") as Model
		val component = model.elements.findFirst([it instanceof Component]) as Component
		
		fsa.generateFile(component.backingBeanFileName(true), GeneratorConstants::OUTLET_GENERATED_JAVA, toAbstractComponentBackingBean(model.name, component))
		logger.info("Generate " + component.backingBeanFileName(true))
		
		fsa.generateFile(component.backingBeanFileName(false),GeneratorConstants::OUTLET_JAVA, toComponentBackingBean(model.name, component))
		logger.info("Generate " + component.backingBeanFileName(false))

				
		val resourcePropertyContent = new StringBuffer();
		val xmaDslPages = component.pages.filter(typeof(XmadslPage))
		xmaDslPages.forEach([page|
			var pageContents = facelet.createContents(page)
			fsa.generateFile(page.faceletPageFileName(true), GeneratorConstants::OUTLET_GENERATED_WEBAPP, pageContents)
			logger.info("Generate " + page.faceletPageFileName(true))
			
			//add properties of the current page.
			resourcePropertyContent.append(generateResourceBundleForPage(page))
						
			fsa.generateFile(page.faceletPageFileName(false), GeneratorConstants::OUTLET_WEBAPP, page.toFaceletPage)
			logger.info("Generate " + page.faceletPageFileName(false))
			
			fsa.generateFile(page.backingBeanFileName(true), GeneratorConstants::OUTLET_GENERATED_JAVA, toAbstractBackingBean(model.name, page, component))
			logger.info("Generate " + page.backingBeanFileName(true))
			
			fsa.generateFile(page.backingBeanFileName(false),GeneratorConstants::OUTLET_JAVA, toBackingBean(model.name, page))
			logger.info("Generate " + page.backingBeanFileName(false))
			
			if (GeneratorConfigurationExtension::automationTestingEnabled) {
				fsa.generateFile(automationTestCaseGenerator.getITCaseName(page), GeneratorConstants::OUTLET_TEST, 
					automationTestCaseGenerator.generateTestCase(page, model.name));
				logger.info("Generate " + automationTestCaseGenerator.getITCaseName(page))
					
				fsa.generateFile(automationTestCaseGenerator.getPageObjectName(page, true), GeneratorConstants::OUTLET_GENERATED_TEST, 
					automationTestCaseGenerator.getAbstractPageObject(page, model.name, facelet.getFieldClientIds(page)));
				logger.info("Generate " + automationTestCaseGenerator.getPageObjectName(page, true))	
					
				fsa.generateFile(automationTestCaseGenerator.getPageObjectName(page, false), GeneratorConstants::OUTLET_TEST, 
					automationTestCaseGenerator.getPageObject(page, model.name));			
				logger.info("Generate " + automationTestCaseGenerator.getPageObjectName(page, false))		
			} 
		])
		
		fsa.generateFile(toResourceBundleFilePath(component),GeneratorConstants::OUTLET_GENERATED_RESOURCES, resourcePropertyContent.toString())
		logger.info("Generate " + toResourceBundleFilePath(component))
	}
	
	def CharSequence toResourceBundle() {'''
	#Enter you own property messages here, property declared here can be referred using msgs variable	
	'''
	}
	
	def CharSequence generateResourceBundleForPage(XmadslPage page) {
		var resourceProperties = facelet.getPageResourceProperties(page)
		var content = ''
		
		if (resourceProperties != null && resourceProperties.size() > 0) {
			content = '''		
				#*************************** Resources for page.name *******************************
				generateResourceBundleFromMap(resourceProperties)
				
			'''
		}
		
		resourceProperties.clear()
		
		return content	
	}
	
	def CharSequence generateResourceBundleFromMap(Map<CharSequence, CharSequence> resourceProperties) {
		val sb = new StringBuffer()
		resourceProperties.forEach([key, value| sb.append(key + "=" + value + "\n")])	 
		
		return sb
	}
	
	def modify(ResourceSet resourceSet) {
		
        val List<Resource> resources = Lists::newArrayList(resourceSet.resources);
        for (Resource resource : resources) {
            if (resource.getEObject("/") instanceof Model && !resource.hasProperty(GeneratorConstants::MODIFIED_MODEL)) {
            	    resource.setProperty(GeneratorConstants::MODIFIED_MODEL, Boolean::TRUE);
            	    logger.info("Pre-process model '" + resource.getURI().lastSegment() + "'");
                generatorConfiguration.modelModifier.modifyModel(resource.getEObject("/") as Model)
            }
        }
    }
	
	def CharSequence toFaceletPage(XmadslPage page) { '''
<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE html
        PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
        "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml"
	xmlns:f="http://java.sun.com/jsf/core"
	xmlns:h="http://java.sun.com/jsf/html"
	xmlns:ui="http://java.sun.com/jsf/facelets"
	xmlns:p="http://primefaces.org/ui"
	xmlns:fn="http://java.sun.com/jsp/jstl/functions">
<head>

</head>
<body>
<ui:composition template="page.name.toFirstLower().gen.xhtml">
	<ui:param name="template" value="/templates/template.xhtml" />
</ui:composition>
</body>
</html>
'''	
	}
}
	
class IGeneratorDelegate implements IFileSystemAccess {
	val logger = getLog(typeof(XtendDomGenerator).name);
	val IFileSystemAccess delegate
	val Notifier notifier

	new(IFileSystemAccess delegate, Notifier notifier) {
		this.delegate = delegate
		this.notifier = notifier
	}

	override deleteFile(String fileName) {
		delegate.deleteFile(fileName)
	}

	override generateFile(String fileName, CharSequence contents) {
		generateFile(fileName, DEFAULT_OUTPUT, contents)
	}

	override generateFile(String fileName, String outputConfigurationName, CharSequence contents) {
		logger.info("Write " + fileName)
		delegate.generateFile(fileName, outputConfigurationName, ImportAdapter::resolve(notifier, contents.toString))
	}
}
