package org.openxma.dsl.generator.impl

import com.google.inject.Singleton
import java.beans.Introspector
import org.eclipse.emf.ecore.EObject
import org.openxma.dsl.pom.model.CheckBox
import org.openxma.dsl.pom.model.Combo
import org.openxma.dsl.pom.model.Command
import org.openxma.dsl.pom.model.Component
import org.openxma.dsl.pom.model.ListBox
import org.openxma.dsl.pom.model.ObjectProperty
import org.openxma.dsl.pom.model.TabFolder
import org.openxma.dsl.pom.model.Table
import org.openxma.dsl.pom.model.TableColumn
import org.openxma.dsl.pom.model.TableCombo
import org.openxma.dsl.pom.model.Text
import org.openxma.dsl.pom.model.XmadslComposite
import org.openxma.dsl.pom.model.XmadslPage
import org.openxma.dsl.pom.model.ReferencedXMAComposite
import org.openxma.dsl.pom.model.SetOfGuiElements
import org.openxma.dsl.pom.model.TabPage
import org.openxma.dsl.pom.model.Content
import org.openxma.dsl.pom.model.Tree
import org.openxma.dsl.pom.model.Button
import org.openxma.dsl.pom.model.CustomizeableField
import java.util.Map
import java.util.HashMap

@Singleton
class JsfNamesImpl implements JsfNames {
	
	def dispatch CharSequence getJSClass(Component component) {
		component.xmaComponent.namClass
	} 
	def dispatch CharSequence getJSClass(XmadslPage page) { 
		page.name
	}
	
	override CharSequence getBean(XmadslPage page) { 
		page.name.toFirstLower
	}
	override CharSequence getDialogWidgetVar(XmadslPage page) { 
		page.name.toFirstLower + "Var"

	}
	def dispatch CharSequence getWidgetVar(EObject page) { 
		throw new RuntimeException("No widget var for this element type!");
	}
	def dispatch CharSequence getWidgetVar(Table table) {
		table.parentPage.JSClass + "_" + table.name + "WidgetVar"
	}
	def dispatch CharSequence getWidgetVar(Component component) { 
		component.xmaComponent.namClass.toFirstLower
	}
	def dispatch CharSequence getWidgetVar(XmadslPage page) { 
		page.name.toFirstLower
	}
	def dispatch CharSequence getWidgetVar(ObjectProperty property) {
		property.parentPage.JSClass + "_" + property.name + "Prop"
	}
	def dispatch CharSequence getWidgetVar(Command command) {
		if (command.eContainer instanceof Component)
			(command.eContainer as Component).JSClass + "_" + command.name + "Cmd"
		else
			command.parentPage.JSClass + "_" + command.name + "Cmd"
	}
	
	
	def dispatch CharSequence referencedBean(Component component) { 
		component.xmaComponent.namClass.toFirstLower
	}
	def dispatch CharSequence referencedBean(XmadslPage page) {
		page.name.toFirstLower
	}
	
	override Component component(XmadslPage page) {
		page.eContainer as Component
	}
	override XmadslPage mainPage(Component component) {
		component.pages?.head as XmadslPage
	}
	
	override String getBackingBeanVar(CheckBox box) {
		box.name + "Selected"
	}
	override String getBackingBeanVar(Text text) {
		text.name + "Text"
	}
	override String getBackingBeanVar(Combo combo) {
		combo.name + "Combo"
	}
	override String getBackingBeanVar(ListBox listbox) {
		listbox.name + "ListBox"
	}
	override String getBackingBeanVar(TableCombo combo) {
		combo.name + "TableCombo"
	}
	
	override XmadslPage getParentPage(TableColumn tableColumn) {
		var EObject parent = tableColumn.eContainer
		do {
			parent = parent.eContainer
		} while (!(parent instanceof XmadslPage))
		parent as XmadslPage
	}
	override XmadslPage getParentPage(EObject container) {
		var EObject parent = container
		while (!(parent instanceof XmadslPage))
			parent = parent.eContainer
		parent as XmadslPage
	}
	
	/**
	 * Takes care of JSF specific capitalization
	 * @see http://www.rqna.net/qna/uwzkp-jsf-2-managed-beans-property-not-found.html
	 * @see http://www.oracle.com/technetwork/java/javase/documentation/spec-136004.html
	 */
	override getFaceletName(String n) {
		Introspector::decapitalize(n.toFirstUpper)
	}
	

	override getFaceletPageName(XmadslPage page, boolean generated) {
		"pages/"+(page.eContainer as Component).xmaComponent.namClass + "/"+page.name.toFirstLower()+ (if (generated)  ".gen" else "")+".xhtml"
	}
	
	
	override CharSequence toFaceletFormId(XmadslPage page) {
		page.name + "Form"  
	}
	def dispatch CharSequence toFaceletId(EObject o) {
		null  
	}
	def dispatch CharSequence toFaceletId(Button button) { 
		button.name		
	} 
	def dispatch CharSequence toFaceletId(CheckBox checkBox) { 
		checkBox.name + "1"		
	} 
	def dispatch CharSequence toFaceletId(XmadslComposite composite) { 
		composite.name		
	} 
	def dispatch CharSequence toFaceletId(Table table) { 
		table.name		
	} 
	def dispatch CharSequence toFaceletId(Tree tree) { 
		tree.name		
	} 
	def dispatch CharSequence toFaceletId(Text text) { 
		text.name + "1"		
	} 
	def dispatch CharSequence toFaceletId(Combo combo) { 
		combo.name + "1"		
	} 
	def dispatch CharSequence toFaceletId(CustomizeableField field) { 
		field.attribute.name + "1"
	} 
	def dispatch CharSequence toFaceletId(TableCombo combo) { 
		combo.name + "1"		
	} 
	def dispatch CharSequence toFaceletId(ListBox box) { 
		box.name + "1"		
	} 
	def dispatch CharSequence toFaceletId(ReferencedXMAComposite composite) { 
		composite.xmaComposite.namInstance		
	} 
	def dispatch CharSequence toFaceletId(TabFolder tabFolder) { 
		tabFolder.name
	} 
	def dispatch CharSequence toFaceletId(TabPage page) {  
		page.name
	} 
	def dispatch CharSequence toFaceletId(Content content) { 
		null
	} 
	
}