package org.openxma.dsl.generator.impl

import com.google.common.collect.Sets
import com.google.inject.Singleton
import java.util.Set
import javax.inject.Inject
import org.eclipse.emf.ecore.EObject
import org.eclipse.xtext.util.SimpleAttributeResolver
import org.openxma.dsl.core.model.DataTypeAndTypeParameter
import org.openxma.dsl.core.model.EnumType
import org.openxma.dsl.core.model.ModelElement
import org.openxma.dsl.core.model.SimpleType
import org.openxma.dsl.dom.model.ApplicationSession
import org.openxma.dsl.dom.model.Dao
import org.openxma.dsl.dom.model.DataView
import org.openxma.dsl.dom.model.Entity
import org.openxma.dsl.dom.model.Service
import org.openxma.dsl.generator.lib.ImportAdapter
import org.openxma.dsl.generator.helper.JavaUtilExtension

@Singleton
class NamesImpl implements Names {
	@Property
	val Set<String> qualifiedNames = Sets::newHashSet
	@Inject
	extension Types
	
	override toVariableName(EObject eObject) {
		var name = SimpleAttributeResolver::NAME_RESOLVER.apply(eObject);
		name?.toVariableName
	}
	
	override toVariableName(String name) {
		JavaUtilExtension::toVariableName(name)
	}
	
	def dispatch String getQualifiedName(DataTypeAndTypeParameter it) {
		getQualifiedName(dataType)
	}

	def dispatch String getQualifiedName(EnumType it) {
		addImport(packageName + "." + name)
	}

	def dispatch String getQualifiedName(SimpleType it) {
		if (custom && typeDefinition.definition != null) {
			addImport(packageName + "." + name)
		} else {
			addImport(getInstanceTypeFullyQualifiedName)
		}
	}

	def dispatch String getQualifiedName(ModelElement it) {
		addImport(packageName + "." + name)
	}
	
	
	override getPackageName(ModelElement modelElement) {
		val model = modelElement.model
		var packageSuffix = switch modelElement {
			Dao: ".dao"
			DataView: ".dto"
			Entity: ".model"
			Service: ".service"
			ApplicationSession: ".context"
			SimpleType case modelElement.custom: ".valueobject"
			default: ""
		}
		if (model.name.endsWith(packageSuffix)) {
			return model.name
		} else {
			model.name + packageSuffix
		}
	}
	
	override getClassPackageName(ModelElement it) {
		switch it {
			Entity: it.packageName + ".impl"
			Dao: it.packageName + ".impl"
			Service: it.packageName + ".impl"
			ApplicationSession: it.packageName + ".impl"
		    default : it.packageName
		}
	}
	
	override getGenInterfaceName(ModelElement it) {
		switch it {
			Entity: it.name + "Gen"
			Dao: it.name + "Gen"
			Service: it.name + "Gen"
			ApplicationSession: it.name + "Gen"
            default : it.name
		}
	}
	
	override getInterfaceName(ModelElement it) {
		name
	}
	
	override getGenClassName(ModelElement it) {
		switch it {
			Entity: it.name + "GenImpl"
			Dao: it.name + "GenImpl"
			Service: it.name + "GenImpl"
			ApplicationSession: it.name + "GenImpl"
			DataView: it.name + "Gen"
            default : it.name
		}
	}

	override getClassName(ModelElement it) {
		switch it {
			Entity: it.name + "Impl"
			Dao: it.name + "Impl"
			Service: it.name + "Impl"
			ApplicationSession: it.name + "Impl"
            default : it.name
		}
	}

	override getQualifiedGenInterfaceName(ModelElement it) {
		packageName + "." + genInterfaceName
	}

	override getQualifiedInterfaceName(ModelElement it) {
		packageName + "." + interfaceName
	}

	override getQualifiedGenClassName(ModelElement it) {
		classPackageName + "." + genClassName
	}

	override getQualifiedClassName(ModelElement it) {
		switch it {
			SimpleType:
				if (it.custom && it.typeDefinition.definition != null) {
					it.packageName + "." + it.name
				} else {
					it.getInstanceTypeFullyQualifiedName
				}
		default : it.classPackageName + "." + it.className
		}
	}

	override addImport(EObject eObject, String qualifiedName) {
		ImportAdapter::addImport(eObject, qualifiedName)
	}
	
	
	
}